{$APPTYPE CONSOLE}

//  Compare two 24-bit bitmaps and write statistical results to a .DAT file.
//  efg, April 1998.

// Compile in DOS Windows:  DCC32 BitmapCompare

PROGRAM BitmapCompare;

  USES
    Windows,             // TRGBTriple
    SysUtils,            // Exception
    Graphics,            // TBitmap
    StatisticsLibrary;   // TDescriptiveStatistics

  CONST
    PixelMaxCount = 65536;

  TYPE
    TRGBTripleArray = ARRAY[0..PixelMaxCount-1] OF TRGBTriple;
    pRGBArray = ^TRGBTripleArray;

    EFatalError = CLASS(Exception);

  VAR
    Bitmap1                :  TBitmap;
    Bitmap2                :  TBitmap;
    CorrelationBlue        :  TLinearRegression;
    CorrelationGreen       :  TLinearRegression;
    CorrelationRed         :  TLinearRegression;
    CorrelationIntensity   :  TLinearRegression;
    CorrelationFile        :  TextFile;
    DeltaBlue              :  DOUBLE;
    DeltaGreen             :  DOUBLE;
    DeltaRed               :  DOUBLE;
    DeltaDistance          :  DOUBLE;
    i                      :  INTEGER;
    j                      :  INTEGER;
    OutFileName            :  STRING;
    row1                   :  pRGBArray;
    row2                   :  pRGBArray;
    StatisticsBlueAbsolute :  TDescriptiveStatistics;
    StatisticsBlueRelative :  TDescriptiveStatistics;
    StatisticsFile         :  TextFile;
    StatisticsGreenAbsolute:  TDescriptiveStatistics;
    StatisticsGreenRelative:  TDescriptiveStatistics;
    StatisticsRedAbsolute  :  TDescriptiveStatistics;
    StatisticsRedRelative  :  TDescriptiveStatistics;
    StatisticsDistance     :  TDescriptiveStatistics;

BEGIN

  IF   (ParamCount <> 3) AND (ParamCount <> 4)
  THEN BEGIN
    WRITELN;
    WRITELN ('Syntax:  BitmapCompare filespec1 filespec2 tag [statistics.dat]');
    WRITELN;
    WRITELN ('where filespec1 and filespec2 are the names of two 24-bit bitmaps.');
    WRITELN ('The output file is named Statistics.DAT unless specified.');
    WRITELN ('The name of the output file Correlation.DAT cannot be changed.');
    WRITELN
  END
  ELSE BEGIN

    IF   NOT FileExists(ParamStr(1))
    THEN RAISE EFatalError.Create('File 1 cannot be found "' + ParamStr(1) + '"');
    Bitmap1 := TBitmap.Create;
    Bitmap1.LoadFromFile(ParamStr(1));

    IF   NOT FileExists(ParamStr(2))
    THEN RAISE EFatalError.Create('File 2 cannot be found "' + ParamStr(2) + '"');
    Bitmap2 := TBitmap.Create;
    Bitmap2.LoadFromFile(ParamStr(2));

    IF  (Bitmap1.Width  <> Bitmap2.Width)  OR
        (Bitmap1.Height <> Bitmap2.Height)
    THEN RAISE EFatalError.Create('Bitmaps do not have the same dimensions:  ' +
                                  IntToStr(Bitmap1.Width) + '-by-' + IntToStr(Bitmap1.Height) + ' versus ' +
                                  IntToStr(Bitmap2.Width) + '-by-' + IntToStr(Bitmap2.Height) );


    IF  (Bitmap1.PixelFormat <> pf24Bit) OR
        (Bitmap2.PixelFormat <> pf24Bit)
    THEN RAISE EFatalError.Create('Only 24-bit bitmaps supported');

    IF   ParamCount = 3
    THEN OutFilename := 'Statistics.Dat'
    ELSE OutFilename := ParamStr(4);

    AssignFile(StatisticsFile, OutFilename);
    AssignFile(CorrelationFile, 'Correlation.DAT');


    IF   FileExists(OutFilename)
    THEN BEGIN
      Append(StatisticsFile);
      Append(CorrelationFile)
    END
    ELSE BEGIN
      REWRITE(StatisticsFile);

      WRITE(StatisticsFile,   '          ');
      WRITE(StatisticsFile,   '<----------------- Red -----------------> ');
      WRITE(StatisticsFile,   '<---------------- Green ----------------> ');
      WRITE(StatisticsFile,   '<----------------- Blue ----------------> ');
      WRITELN(StatisticsFile, '<----- RGB Composite ----->');

      WRITE(StatisticsFile,   '          ');
      WRITE(StatisticsFile,   '    Absolute      Relative                ');
      WRITE(StatisticsFile,   '    Absolute      Relative                ');
      WRITE(StatisticsFile,   '    Absolute      Relative                ');
      WRITELN(StatisticsFile, '    Absolute');


      WRITE(StatisticsFile,   '   ID     ');
      FOR i := 1 TO 3 DO
      BEGIN
        WRITE(StatisticsFile, Format(' %5s  %5s  %5s  %5s  %5s %5s |',
              [' mean', ' S.D.', ' mean', ' S.D.', ' min ', ' max ']))
      END;
      WRITELN(StatisticsFile,
            Format(' %5s  %5s  %5s  %5s', [' mean', ' S.D.', ' min ', ' max ']));

      WRITE(StatisticsFile,   '--------  ');
      FOR i := 1 TO 3 DO
      BEGIN
        WRITE(StatisticsFile, Format(' %5s  %5s  %5s  %5s  %5s %5s |',
              ['-----', '-----', '-----', '-----', '-----', '-----']))
      END;
      WRITELN(StatisticsFile,
            Format(' %5s  %5s  %5s  %5s', ['-----', '-----', '-----', '-----']));


      REWRITE(CorrelationFile);
      WRITELN(CorrelationFile,  '   ID       Red     Green    Blue      RGB');
      WRITELN(CorrelationFile,  '--------  -------  -------  -------  -------')
    END;

    StatisticsRedAbsolute      := TDescriptiveStatistics.Create;
    StatisticsGreenAbsolute    := TDescriptiveStatistics.Create;
    StatisticsBlueAbsolute     := TDescriptiveStatistics.Create;

    StatisticsRedRelative      := TDescriptiveStatistics.Create;
    StatisticsGreenRelative    := TDescriptiveStatistics.Create;
    StatisticsBlueRelative     := TDescriptiveStatistics.Create;

    StatisticsDistance := TDescriptiveStatistics.Create;

    CorrelationRed       := TLinearRegression.Create;
    CorrelationGreen     := TLinearRegression.Create;
    CorrelationBlue      := TLinearRegression.Create;
    CorrelationIntensity := TLinearRegression.Create;

    TRY

      FOR j := 0 TO Bitmap1.Height-1 DO
      BEGIN
        row1       := Bitmap1.ScanLine[j];
        row2       := Bitmap2.ScanLine[j];

        FOR i := 0 TO Bitmap1.Width-1 DO
        BEGIN
          CorrelationRed.NextValue(row1[i].rgbtRed,  row2[i].rgbtRed);
          CorrelationGreen.NextValue(row1[i].rgbtGreen,  row2[i].rgbtGreen);
          CorrelationBlue.NextValue(row1[i].rgbtBlue,  row2[i].rgbtBlue);
          CorrelationIntensity.NextValue((row1[i].rgbtRed + row1[i].rgbtGreen + row1[i].rgbtBlue) / 3,
                                         (row2[i].rgbtRed + row2[i].rgbtGreen + row2[i].rgbtBlue) / 3);

          DeltaRed      := INTEGER(row1[i].rgbtRed)   - INTEGER(row2[i].rgbtRed);
          DeltaGreen    := INTEGER(row1[i].rgbtGreen) - INTEGER(row2[i].rgbtGreen);
          DeltaBlue     := INTEGER(row1[i].rgbtBlue)  - INTEGER(row2[i].rgbtBlue);
          DeltaDistance := SQRT( SQR(DeltaRed) + SQR(DeltaGreen) + SQR(DeltaBlue) );

          StatisticsRedRelative.NextValue(DeltaRed);           // -255 .. 255
          StatisticsGreenRelative.NextValue(DeltaGreen);       // -255 .. 255
          StatisticsBlueRelative.NextValue(DeltaBlue);         // -255 .. 255

          StatisticsRedAbsolute.NextValue(ABS(DeltaRed));      // 0 .. 255
          StatisticsGreenAbsolute.NextValue(ABS(DeltaGreen));  // 0 .. 255
          StatisticsBlueAbsolute.NextValue(ABS(DeltaBlue));    // 0 .. 255

          StatisticsDistance.NextValue(DeltaDistance);         // 0 .. SQRT(255^3)

        END

      END;

      WRITE(StatisticsFile, ParamStr(3):8, '  ');   // "ID" field



      WRITE(StatisticsFile,

            Format(' %5.1f  %5.1f  %5.1f  %5.1f  %5.0f %5.0f |',
                   [StatisticsRedAbsolute.MeanValue,
                    StatisticsRedAbsolute.StandardDeviation,
                    StatisticsRedRelative.MeanValue,
                    StatisticsRedRelative.StandardDeviation,
                    StatisticsRedRelative.MinValue,
                    StatisticsRedRelative.Maxvalue]));

      WRITE(StatisticsFile,
            Format(' %5.1f  %5.1f  %5.1f  %5.1f  %5.0f %5.0f |',
                   [StatisticsGreenAbsolute.MeanValue,
                    StatisticsGreenAbsolute.StandardDeviation,
                    StatisticsGreenRelative.MeanValue,
                    StatisticsGreenRelative.StandardDeviation,
                    StatisticsGreenRelative.MinValue,
                    StatisticsGreenRelative.Maxvalue]));

      WRITE(StatisticsFile,
            Format(' %5.1f  %5.1f  %5.1f  %5.1f  %5.0f %5.0f |',
                   [StatisticsBlueAbsolute.MeanValue,
                    StatisticsBlueAbsolute.StandardDeviation,
                    StatisticsBlueRelative.MeanValue,
                    StatisticsBlueRelative.StandardDeviation,
                    StatisticsBlueRelative.MinValue,
                    StatisticsBlueRelative.Maxvalue]));

      WRITELN(StatisticsFile,
            Format(' %5.1f  %5.1f  %5.0f  %5.0f',
                   [StatisticsDistance.MeanValue,
                    StatisticsDistance.StandardDeviation,
                    StatisticsDistance.MinValue,
                    StatisticsDistance.Maxvalue]));

      WRITELN(CorrelationFile,
              Format('%8s %8.5f %8.5f %8.5f %8.5f',
                     [ParamStr(3), CorrelationRed.CorrelationCoefficient,
                                   CorrelationGreen.CorrelationCoefficient,
                                   CorrelationBlue.CorrelationCoefficient,
                                   CorrelationIntensity.CorrelationCoefficient]));

    FINALLY
      StatisticsRedAbsolute.Free;
      StatisticsGreenAbsolute.Free;
      StatisticsBlueAbsolute.Free;

      StatisticsRedRelative.Free;
      StatisticsGreenRelative.Free;
      StatisticsBlueRelative.Free;

      StatisticsDistance.Free;

      CorrelationRed.Free;
      CorrelationGreen.Free;
      CorrelationBlue.Free;
      CorrelationIntensity.Free;

      Bitmap1.Free;
      Bitmap2.Free;

      CloseFile(StatisticsFile);
      CloseFile(CorrelationFile)
    END


  END
END.
