unit G32_Interface;

{ native rus   (rus) :        GR32 }
{ poor english (eng) : some usefull routines and additional buildups to GR32}
{ public 23.01.2002  (c) roma@goodok.ru }
{ version <= 0.1  }

interface

uses GR32, GR32_Polygons, Graphics,
     windows, classes, GR32_Transforms;

{.$DEFINE TextFitting_DEMO}
{ tPolygonDrawOptions }

  { rus:     }
  { end: this type usefull to point draw method }

const
  pdoAntialising = 1;  { c }
  pdoFloat       = 2 or pdoAntialising;   {       -     }
  pdoFilling     = 4; {   }
  pdoFastFilling = 8 or pdoFilling or pdoFloat; {    -    ,
                                                    }



type
  tPolygonDrawOptions = longint;


{ rus: ,     }
{ eng: quality of curve approximation }
const
   { eng: for qubic bezier curve }
   Bezier3SegmentMinLengthInPixel : word = 8; { rus:        Bezier,
                                                      }
   { eng: for quadric bezier curve  (QSpline_ }
   Bezier2SegmentMinLengthInPixel : word = 10; { rus:        Bezier,
                                                      }

const
  Identity_mat2 : tmat2 = ( eM11 : (fract : 0; value : 1);
                            eM12 : (fract : 0; value : 0);
                            eM21 : (fract : 0; value : 0);
                            eM22 : (fract : 0; value : 1); );

  VertFlip_mat2 : tmat2 = ( eM11 : (fract :  0; value : 1);
                            eM12 : (fract :  0; value : 0);
                            eM21 : (fract :  0; value : 0);
                            eM22 : (fract :  0; value : -1); );

  {  G32   Win32 API   }
  function FloatMatrixToMat2(xMat : tFloatmatrix) : TMAT2;

  function WinFixToFixed(x : _FIXED) : GR32.TFixed;
  function FixedToWinFix(x : GR32.TFixed) : _Fixed;

  function FloatToWinFix(x : single) : _FIXED;
  function WinFixToFloat(x : _Fixed) : single;



  { rus:  :  }
  function FixedRect(xR : tRect) : tFixedRect;
  {eng: average }
  function Mdl(X, Y : integer) : integer; {c   }
  function MiddleLine(const p1, p2 : tFixedPoint) : tFixedPoint;
  function PointsAreEqual(p1, p2 : GR32.tFixedPoint) : boolean;
  function Distance(p1, p2 :GR32.tFixedPoint) : GR32.tFixed; overload;{    }
  function Distance(p1x, p1y, p2x, p2y : double) : double; overload;{    }
  function SqrDistance(p1, p2 :GR32.tFixedPoint) : GR32.tFixed; {     }
  function SegmentLengthValuation(p1, p2, p3, p4 : tFixedPoint) : GR32.tFixed; {      }

  { conditional distance }
  function Norma(x1, y1, x2, y2: integer) : integer; overload; {  }
  function Norma(p1, p2 : tFixedPoint) : GR32.tFixed; overload;


  { rus:   -           }
  { eng: for debug and diagnostic }
  function g32i_GetCurvePointsCount : integer;
  { rus:   -         ""   }
  { eng: for debug and diagnostic }
  function g32i_GetCurvePointsMaxCount : integer;

  procedure gPolygon(Bitmap: TBitmap32;
                     const Points: TArrayOfFixedPoint;
                     Color: TColor32;
                     Options : tPolygonDrawOptions;
                     Closed: Boolean;
                     FillMode: TPolyFillMode = pfAlternate);

  procedure gPolyPolygon(Bitmap: TBitmap32;
                     const Points: TArrayOfArrayOfFixedPoint;
                     Color: TColor32;
                     Options : tPolygonDrawOptions;
                     Closed: Boolean;
                     FillMode: TPolyFillMode = pfAlternate);

  procedure gPolyBezier(Bitmap: TBitmap32;
                       const Points: TArrayOfFixedPoint;
                       Color: TColor32;
                       Options : tPolygonDrawOptions;
                       Closed: Boolean;
                       FillMode: TPolyFillMode = pfAlternate);

  procedure gPolyPolyBezier(Bitmap: TBitmap32;
                     const Points: TArrayOfArrayOfFixedPoint;
                     Color: TColor32;
                     Options : tPolygonDrawOptions;
                     Closed: Boolean;
                     FillMode: TPolyFillMode = pfAlternate);


{ rus:     tBitmap32,        }
{ end: wrapper around tBitmap32, to don't depend on future versions g32 lib. }
type

  TBitmap32Ex = class(tBitmap32)
  private
    fFont_e31 : Single;
    fFont_e32 : Single;
    function GetCanvas: tCanvas;
  protected
    fDrawOrign : tFixedPoint;
    fCanvas    : tCanvas;
    fFontMat2  : tMat2;
  public
    constructor Create; override;
    destructor Destroy; override;

    procedure Polygon(const Points: TArrayOfFixedPoint;   Color: TColor32;    Options : tPolygonDrawOptions;
                     Closed: Boolean;                     FillMode: TPolyFillMode = pfAlternate);
    procedure PolyBizer(const Points: TArrayOfFixedPoint; Color: TColor32; Options : tPolygonDrawOptions;
                       Closed: Boolean; FillMode: TPolyFillMode = pfAlternate);


    procedure Ellipse(xRect : TFixedRect; xColor: TColor32; xOptions : tPolygonDrawOptions);
    procedure PolyPolygon(const Points : TArrayOfArrayOfFixedPoint; Color : tColor32; Options : tPolygonDrawOptions;
                     Closed: Boolean; FillMode: TPolyFillMode = pfAlternate);
    procedure PolyPolyBezier(const Points : TArrayOfArrayOfFixedPoint; Color : tColor32; Options : tPolygonDrawOptions;
                     Closed: Boolean; FillMode: TPolyFillMode = pfAlternate);


    { rus:       Font     }
    { eng : draw glyph of one symbol of current font in position (xLeft, yTop) }
    procedure  DrawGlyph(xCharCode : longword;
                         xLeft, yTop : GR32.tFixed;
                         xColor : tColor32; xOptions : tPolygonDrawOptions);

    {    ; xLeft, yTop -   __ }
    procedure RenderTextEx(xLeft, yTop: GR32.tFixed; const xText: string;
                                xColor : tColor32; xOptions : tPolygonDrawOptions);

    procedure GetPointAtSegment(xSegment: tArrayOfFixedPoint; xLength : GR32.tFixed; out xPathX, xPathY : GR32.tFixed; out xAngle : double); virtual;
    {     xPath}
    procedure RenderFittedText(const xText : string; xColor : tColor32; xOptions : tPolygonDrawOptions;
                               xPath : tArrayOfFixedPoint);

    {      (       )  }
    function SelectFontMat2(xValue : tMat2) : tMat2;
    function SelectFontTransform(xValue : tFloatMatrix) : tFloatMatrix; {     }

    { TODO : DrawComplexCurve -  ,   ,          }

    { eng : true canvas }
    property Canvas : tCanvas read GetCanvas;
    { eng : orign of viewport }
    property DrawOrign : tFixedPoint read fDrawOrign write fDrawOrign;  {    (!)      }
  end;



implementation

  uses Sysutils, Math{, Utils};

  function FixedRect(xR : tRect) : tFixedRect;
  begin
    result.Left := GR32.fixed(xR.left);
    result.Right := GR32.fixed(xR.right);
    result.Top := GR32.fixed(xR.top);
    result.Bottom := GR32.fixed(xR.bottom);
  end;

  function FloatToWinFix(x : single) : _FIXED;
  begin
    result := _Fixed(GR32.tFixed(Round(x * 65536)));
  end;

  function WinFixToFloat(x : _Fixed) : single;
  begin
    result := GR32.tFixed(x)/65536;
  end;

  function WinFixToFixed(x : _FIXED) : GR32.TFixed;
  begin
    result := GR32.TFixed(x);
  end;

  function FixedToWinFix(x : GR32.TFixed) : _Fixed;
  begin
    result := _Fixed(x);
  end;

  {  G32   Win32 API   }
  function  FloatMatrixToMat2(xMat : tFloatMatrix) : TMAT2;
  begin
    result.eM11 := FloatToWinFix(xMat[0,0]);
    result.eM21 := FloatToWinFix(xMat[1,0]);
    result.eM12 := FloatToWinFix(xMat[0,1]);
    result.eM22 := FloatToWinFix(xMat[1,1]);
  end;

  {  Win32Api    g32-}
  function Mat2ToFloatMatrix(xMat : tMat2) : tFloatMatrix;
  begin
    result[0,0] := WinFixToFloat(xMat.eM11);
    result[1,0] := WinFixToFloat(xMat.eM21);
    result[0,1] := WinFixToFloat(xMat.eM12);
    result[1,1] := WinFixToFloat(xMat.eM22);
  end;

  {    }
  function GetRotatedMat2(xAngle : double) : tMat2;
  var
    S, C : single;
  begin
    S := Sin(xAngle); C := Cos(xAngle);
    result.eM11 := FloatToWinFix(C);
    result.eM21 := FloatToWinFix(S);
    result.eM12 := FloatToWinFix(-S);
    result.eM22 := FloatToWinFix(C);
  end;

  function MultMat2(const M1, M2: tMat2) : tmat2;
  var
    m1_00, m1_01, m1_10, m1_11,
    m2_00, m2_01, m2_10, m2_11 : double;
  begin
    m1_00 := WinFixToFloat(m1.eM11);
    m1_01 := WinFixToFloat(m1.eM12);
    m1_10 := WinFixToFloat(m1.eM21);
    m1_11 := WinFixToFloat(m1.eM22);

    m2_00 := WinFixToFloat(m2.eM11);
    m2_01 := WinFixToFloat(m2.eM12);
    m2_10 := WinFixToFloat(m2.eM21);
    m2_11 := WinFixToFloat(m2.eM22);

    Result.eM11 :=  FloatToWinFix(m1_00 * m2_00 +  m1_10 * m2_01);
    Result.eM12 :=  FloatToWinFix(m1_01 * m2_00 +  m1_11 * m2_01);
    Result.eM21 :=  FloatToWinFix(m1_00 * m2_10 +  m1_10 * m2_11);
    Result.eM22 :=  FloatToWinFix(m1_01 * m2_10 +  m1_11 * m2_11);
  end;

  function DetMat2(xMat : tMat2): Single;
  begin
    Result := WinFixToFloat(xMat.eM11) * WinFixToFloat(xMat.eM22) - WinFixToFloat(xMat.eM12) * WinFixToFloat(xMat.eM21);
  end;

  function Mdl(X, Y : integer) : integer;
  asm
    mov eax, x  { result := (x + y) div 2; }
    add eax, y
    sar eax, 1
  end;

  function Norma(x1, y1, x2, y2: integer) : integer;
  begin
    result := abs(y2 - y1) + abs(x2 - x1);
  end;

  function Norma(p1, p2 : tFixedPoint) : GR32.tFixed;
  begin
    result := abs(p2.y - p1.y) + abs(p2.x - p1.x);
  end;

  function MiddleLine(const p1, p2 : tFixedPoint) : tFixedPoint;
  begin
    result.x := (p1.x + p2.x) div 2;
    result.y := (p1.y + p2.y) div 2;
  end;

  function PointsAreEqual(p1, p2 : GR32.tFixedPoint) : boolean;
  begin
    result := (p1.x = p2.x) and (p1.y = p2.y);
  end;

  function Distance(p1, p2 :GR32.tFixedPoint) : GR32.tFixed; {    }
  begin
    result := Round(Distance(p1.x, p1.y, p2.x, p2.y));
  end;

  function Distance(p1x, p1y, p2x, p2y :double) : double; {    }
  begin
    result := sqrt(sqr(p2x - p1x) + sqr(p2y - p1y));
  end;

  function SqrDistance(p1, p2 :GR32.tFixedPoint) : GR32.tFixed; {     }
  begin
    result := sqr(p2.x - p1.x) + sqr(p2.y - p1.y);
  end;

 {      }
 function SegmentLengthValuation(p1, p2, p3, p4 : tFixedPoint) : GR32.tFixed;
 var
   TopValuation, LowValuation : GR32.tFixed;
 begin
   TopValuation := Distance(p1, p2) + Distance(p2, p3) + Distance(p3, p4);
   LowValuation := Distance(p1, p4);
   result := (TopValuation + LowValuation) div 2;
 end;


procedure gPolygon(Bitmap: TBitmap32;
                   const Points: TArrayOfFixedPoint;
                   Color: TColor32;
                   Options : tPolygonDrawOptions;
                   Closed: Boolean;
                   FillMode: TPolyFillMode = pfAlternate);
begin
        if (Options and  pdoFloat)  = pdoFloat then
        begin
        {  - Antialiasing }
        if ByteBool(Options and pdoFilling) then
          begin
          if (Options and  pdoFastFilling) = pdoFastFilling then
            begin
            PolygonTS(Bitmap, Points,   Color, FillMode); // rus:   
                                                          // eng: fill without antialising
            PolylineXS(Bitmap, Points,   Color, true); // rus:   
                                                       // eng: border with antialising
            end
          else
            begin
            PolygonXS(Bitmap, Points, Color, FillMode);
            end;
          end
        else
          begin
          PolylineXS(Bitmap, Points, Color, Closed);
          end
        end
      else
        begin
        { rus:    }
        { eng: use integer-based methods }
        if ByteBool(Options and pdoAntialising) then
          begin
          if ByteBool(Options and pdoFilling) then PolygonXS(Bitmap, Points, Color, FillMode) //    
                                      else PolylineAS(Bitmap, Points,   Color, Closed)
          end
        else
          begin
          if ByteBool(Options and pdoFilling) then PolygonTS(Bitmap, Points, Color, FillMode)
                                      else PolylineTS(Bitmap, Points,   Color, Closed);
          end
        end;
end;

procedure gPolyPolygon(Bitmap: TBitmap32;
                   const Points: TArrayOfArrayOfFixedPoint;
                   Color: TColor32;
                   Options : tPolygonDrawOptions;
                   Closed: Boolean;
                   FillMode: TPolyFillMode = pfAlternate);
begin
        if (Options and  pdoFloat)  = pdoFloat then
        begin
        { rus:  - Antialiasing }
        if ByteBool(Options and pdoFilling) then
          begin
          if (Options and  pdoFastFilling) = pdoFastFilling then
            begin
            PolyPolygonTS(Bitmap, Points,   Color, FillMode); // rus:   
            PolyPolylineXS(Bitmap, Points,   Color, true);    // rus:   
            end
          else
            begin
            PolyPolygonXS(Bitmap, Points, Color, FillMode);
            end;
          end
        else
          begin
          PolyPolylineXS(Bitmap, Points, Color, Closed);
          end
        end
      else
        begin
          { rus:    }
        if ByteBool(Options and pdoAntialising) then
          begin
          if ByteBool(Options and pdoFilling) then PolyPolygonXS(Bitmap, Points, Color, FillMode) //    
                                      else PolyPolylineAS(Bitmap, Points,   Color, Closed)
          end
        else
          begin
          if ByteBool(Options and pdoFilling) then PolyPolygonTS(Bitmap, Points, Color, FillMode)
                                      else PolyPolylineTS(Bitmap, Points,   Color, Closed);
          end
        end;
end;


type
   tRelyativeRgn = (rrLeft, rrRight, rrTop, rrBottom, rrInside);
   { rus:   }
   { eng: relyative situation .. }

  {.. on rectangle }
  function GetRelRgn(xP: tFixedPoint; xR : TFixedRect) : tRelyativeRgn;
  begin
    { rus:  -    ( ) }
    { eng: right side is most probable side}
    if (xP.x > xR.Right) then
       begin
       result := rrRight; exit;
       end;
    if (xP.y > xR.Bottom ) then
       begin
       result := rrBottom;
       exit;
       end;
    if (xP.x < xR.Left) then
       begin
       result := rrLeft;
       exit;
       end;
    if (xP.y < xR.Top) then
       begin
       result := rrTop;
       exit;
       end;
    result := rrInside
  end;



{ rus:  true,   ,      
    ( ):
      ,          
  ,  .
 }

 { eng: return true, if surely kbowns all points lays out of rectangle
 }

function ClipPolyOutside(xPoints : TArrayOfFixedPoint; xR : TFixedRect) : boolean;
var
  OldRgn : tRelyativeRgn;
  i : integer;
begin
  result := true;
  if Length(xPoints) = 0 then exit;

  OldRgn := GetRelRgn(xPoints[0], xR);

  if (OldRgn = rrInside) then
    begin
    result := false;
    exit;
    end;

  if Length(xPoints) = 1 then exit;  { OldRgn <> rrInside }

  for i := 1 to Length(xPoints) - 1 do
    begin
    if OldRgn <> GetRelRgn(xPoints[i], xR) then
      begin
      result := false;
      exit;
      end;
    end;
end;

{ rus:  true,   ,      }
function ClipRectOutside(p0, p1, p2, p3 : tFixedPoint; xR : TFixedRect) : boolean; overload;
var
  OldRgn : tRelyativeRgn;
begin
{  result := true;}

  OldRgn := GetRelRgn(p0, xR);

  if (OldRgn = rrInside) then
    begin
    result := false;
    exit;
    end;

  result := (OldRgn = GetRelRgn(p1, xR)) and (OldRgn = GetRelRgn(p2, xR)) and (OldRgn = GetRelRgn(p3, xR));
end;

{  true,   ,       }
function ClipRectOutside(p0, p1, p2 : tFixedPoint; xR : TFixedRect) : boolean; overload;
var
  OldRgn : tRelyativeRgn;
begin
{  result := true;}

  OldRgn := GetRelRgn(p0, xR);

  if (OldRgn = rrInside) then
    begin
    result := false;
    exit;
    end;

  result := (OldRgn = GetRelRgn(p1, xR)) and (OldRgn = GetRelRgn(p2, xR));
end;


{
  CurvePoints -           LoadApproxCurve;
                            
                   P_xxx
}

{  eng:   urvePoints - is a storage of linear approximation of qubic Bezier curve
}

const
  { rus:      }
  MaxCurvePointsCount  = 512;
var
  CurvePoints : array of TFixedPoint;       {   ,        }
  CurvePointsCount : integer;

  function g32i_GetCurvePointsCount : integer;
  begin
    result := CurvePointsCount;
  end;

  function g32i_GetCurvePointsMaxCount : integer;
  begin
    result := Length(CurvePoints);
  end;

  { eng: pretends only ! }
  {  ,    }
  procedure CP_Clear;
  begin
    CurvePointsCount := 0;
  end;

  procedure CP_Dispose;
  begin
    CP_Clear;
    SetLength(CurvePoints, 0);
  end;

  {   }
  procedure CP_Add(p : tFixedPoint);
  begin
    {  ! }

    CurvePoints[CurvePointsCount] := p;
    inc(CurvePointsCount);
  end;


{     }
{    ,   ,
     (xClipRect)     }
procedure BuildApproxCurve(const xPoints: TArrayOfFixedPoint;
                               xMinSegmentLength : GR32.tFixed;
                               xCliptRect : tFixedRect
                               );
var
  i : integer;

  function SegmentConditionalLength(x0, x1, x2, x3 : tFixedPoint) : GR32.tFixed;
  begin
    result := norma(x0, x1)  + norma(x1, x2) + norma(x2, x3);
  end;

  {   x0, x1, x2, x3     ,   urvePoints[CurvePointsCount]
     
    : x0 -   
             x1, x2 -  ,    ,       
             x3 - ,      

   }
  procedure BreakSegment(p0, p1, p2, p3 : tFixedPoint);
  var
    p11, p21, p31, p22, p32, p33 : tFixedPoint;
  begin
       {        ,   }
       if ((p0.x = p1.x) and (p0.y = p1.y) and (p2.x = p3.x) and (p2.y = p3.y)) then
         begin
         {    }
         CP_Add(p1); CP_Add(p2); CP_Add(p3);
         exit;
         end;

      if ClipRectOutside(p0, p1, p2, p3, xCliptRect) then
        begin
         {    }
         CP_Add(p1); CP_Add(p2); CP_Add(p3);
         exit;
        end;

      {   ,      }
      if (CurvePointsCount >=  Length(CurvePoints) div 2 - 4) then    // ! div 2 -   
        begin
          try
            SetLength(CurvePoints, MaxCurvePointsCount + length(CurvePoints));
          except
            CP_Add(p1); CP_Add(p2); CP_Add(p3); { ?      }
            exit;
          end
        end;

        { 1.          }
        p11 := MiddleLine(p1, p0);      // 1     
        p21 := MiddleLine(p2, p1 );
        p31 := MiddleLine(p3 ,p2 );     // 2     
        p22 := MiddleLine(p11 , p21 );   // 2     
        p32 := MiddleLine(p21 , p31 );   // 1     
        p33 := MiddleLine(p22 , p32);   //   ,       

        {               }
        {  }
        if SegmentConditionalLength(p0, p11, p22, p33) > xMinSegmentLength then
           begin
           BreakSegment(p0, p11, p22, p33)
           end
        else
          begin
          {   }
          CP_Add(p11);
          CP_Add(p22);
          CP_Add(p33);
          end;
        { p33 -   ; : p33   }
        if SegmentConditionalLength(p33, p32, p31, p3) > xMinSegmentLength then
          begin
          BreakSegment(p33, p32, p31, p3)
          end
        else
          begin
          CP_Add(p32);
          CP_Add(p31);
          CP_Add(p3);
          end
  end;

begin
   if Length(xPoints) = 0 then exit;
      CP_Clear;
      i := 0;
      CP_Add(xPoints[0]);
      while i  <= Length(xPoints) - 4 do
        begin
        {    : }
        { (i) - 1, (i+1) - 1  , (i+2) - 2  , (i+3) - c    }
        BreakSegment(xPoints[i], xPoints[i+1], xPoints[i+2], xPoints[i+3]);
        inc(i, 3);
        end;
end;



procedure gPolyBezier(Bitmap: TBitmap32;
                       const Points: TArrayOfFixedPoint;
                       Color: TColor32;
                       Options : tPolygonDrawOptions;
                       Closed: Boolean;
                       FillMode: TPolyFillMode = pfAlternate);

var
  PP : TArrayOfFixedPoint;
  i  : integer;
  PPCount : integer;
begin

  if (Length(Points) <= 1) or ((Length(Points) - 1) mod 3 <> 0) then exit;

  { TODO :    -   -       ,    
        }
  if ClipPolyOutside(Points, FixedRect(rect(0, 0, Bitmap.Width, Bitmap.Height))) then exit;

  { rus:       }
  BuildApproxCurve(Points,  Bezier3SegmentMinLengthInPixel shl 16, FixedRect(rect(0, 0, Bitmap.Width, Bitmap.Height)));

  { rus:    }
  { eng: delete control points }
  PPCount := (CurvePointsCount - 1) div 3 + 1;
  SetLength(PP, PPCount);
  for i := 0 to PPCount - 1 do
    begin
    PP[i] := CurvePoints[3*i];
    end;

  gPolygon(Bitmap,
           PP,
           Color,
           Options,
           Closed,
           FillMode);
end;

procedure gPolyPolyBezier(Bitmap: TBitmap32;
                     const Points: TArrayOfArrayOfFixedPoint;
                     Color: TColor32;
                     Options : tPolygonDrawOptions;
                     Closed: Boolean;
                     FillMode: TPolyFillMode = pfAlternate);

var
  PP : TArrayOfArrayofFixedPoint;
  i, j  : integer;
  PPCount : integer;
begin
  SetLength(PP, Length(Points));

  for j := 0 to Length(Points) - 1 do
    begin
    if (Length(Points[j]) <= 1) or ((Length(Points[j]) - 1) mod 3 <> 0) then continue;

    if ClipPolyOutside(Points[j], FixedRect(rect(0, 0, Bitmap.Width, Bitmap.Height))) then exit;

    { rus:       }
    BuildApproxCurve(Points[j],  Bezier3SegmentMinLengthInPixel shl 16, FixedRect(rect(0, 0, Bitmap.Width, Bitmap.Height)));

    { rus:    }
    { eng: delete control points }
    PPCount := (CurvePointsCount - 1) div 3 + 1;
    SetLength(PP[j], PPCount);
    for i := 0 to PPCount - 1 do
      begin
      PP[j, i] := CurvePoints[3*i];
      end;
    end;

  gPolyPolygon(Bitmap,
           PP,
           Color,
           Options,
           Closed,
           FillMode);
end;

{
  rus: GlyphPolygon -            BuildGlyphPolyPolygon;
                            
                   PathPoints
}

type
    TFXPArray = array[0..0] of TPOINTFX;
    PFXPArray = ^TFXPArray;

var
  GlyphPolygon : TPolygon32; {  ,    }

{ rus :       }
procedure BuildGlyphPolygon(xBufPtr :  PTTPolygonHeader;     {  rus :   ,       }
                            xBufSize : integer;              {  rus :   }
                            xMinSegmentLength : GR32.tFixed; { rus :    ( )      }
                            xLeft, yTop : GR32.tFixed;       { rus :   ,        }
                            xGM : TGLYPHMETRICS;             { rus :          . }
                            xCliptRect : GR32.tFixedRect);   { rus :   }


var
   pc : PTTPolyCurve;
   ps, p1, p2 : TFixedPoint;
   ofs, ofs2, pcSize : LongInt;
   done : boolean;
   i : LongInt;
   pfxA, pfxB, pfXC : TFixedPoint;
   lpAPFX : PFXPArray;
   polyN  : LongInt;
   pcType : LongInt;

   function xWinFixToG32(x : _FIXED) : GR32.TFixed;
   begin
     result := X.value*$10000 + x.fract + xLeft;
   end;

   function yWinFixToG32(y : _FIXED) : GR32.TFixed;
   begin
     result := + y.value* $10000 + y.fract + yTop;
   end;

   { rus:    }
   function SegmentConditionalLength(x0, x1, x2 : tFixedPoint) : GR32.tFixed;
   begin
     result := norma(x0, x1)  + norma(x1, x2);
   end;

   { rus:    }
   function SegmentConditionalCurvature(x0, x1, x2 : tFixedPoint) :GR32.tFixed;
   begin
     result := SegmentConditionalLength(x0, x1, x2);
   end;

  { rus:   p0, p1, p2     ,   urvePoints[CurvePointsCount]
     
    : x0 -      (!)
             p1 -  ,   
             p2 - ,      

   }

  procedure BreakSegment(p0, p1, p2 : tFixedPoint);
  var
    p21, p3, p22 : tFixedPoint;
  begin

       { rus:        ,   }
       if PointsAreEqual(p0, p1) or PointsAreEqual(p1, p2) then
         begin
         { rus:    }
         GlyphPolygon.Add(p1); GlyphPolygon.Add(p2);
         exit;
         end;

      { rus:        ,    }
      if ClipRectOutside(p0, p1, p2, xCliptRect) then
        begin
         { rus:    }
         GlyphPolygon.Add(p1); GlyphPolygon.Add(p2);
         exit;
        end;

        { rus: 1.      2x   }
        p21 := MiddleLine(p1, p0); {rus:        p0-p21-p3}
        p22 := MiddleLine(p2, p1); { rus:       p3 - p22 - p2}
        p3  := MiddleLine(p21, p22);

        { rus:          ( )      }
        if SegmentConditionalCurvature(p0, p21, p3) > xMinSegmentLength then
           begin
           BreakSegment(p0, p21, p3)
           end
        else
          begin
          { rus:   }
          GlyphPolygon.Add(p21);
          GlyphPolygon.Add(p3);
          end;
        { rus: p33 -   ; : p33   }
        if SegmentConditionalCurvature(p3, p22,  p2) > xMinSegmentLength then
          begin
          BreakSegment(p3, p22, p2)
          end
        else
          begin
          GlyphPolygon.Add(p22);
          GlyphPolygon.Add(p2);
          end
  end;

begin
     GlyphPolygon.Clear;
     done := false;
     ofs := 0;
     polyN := 0;

     while not Done do
      begin

           ps.X := xWinFixToG32( xBufPtr^.pfxStart.X );
           ps.Y := yWinFixToG32( xBufPtr^.pfxStart.Y );
           pcSize := xBufPtr^.cb - SizeOf(TTTPOLYGONHEADER);          // rus: ,    /
           pChar(pc) := pChar(xBufPtr) + SizeOf(TTTPOLYGONHEADER);    // rus: pc -      TTPOLYCURVE
           ofs2 := 0;                                                 // rus:     /

           p2 := ps;
           if polyN <> 0 then
             begin
             GlyphPolygon.NewLine;
             end;
          GlyphPolygon.Add(p2);
           { rus:      }
           while not Done and (ofs2 < pcSize) do
            begin
            {rus:     TTPOLYCURVE: ,  -
                 
              wType word -  (/)
              cpfx  word -  
              apfx  -  
            }
                 pcType := pc^.wType;
                 case pcType of
                   TT_PRIM_LINE:
                      begin
                           lpAPFX := @pc^.apfx[0];
                           for i := 0 to pc^.cpfx-1 do
                            begin
                                 p1 := p2;
                                 p2.X := xWinFixToG32(lpAPFX^[i].X);
                                 p2.Y := yWinFixToG32(lpAPFX^[i].Y);
                                 if not PointsAreEqual( p1, p2 )
                                  then GlyphPolygon.Add(p2);
                            end;
                      end;
                    TT_PRIM_QSPLINE:
                      begin
                           lpAPFX := @pc^.apfx[0];
                           pfxA := p2;
                           for i := 0 to pc^.cpfx-2 do
                            begin
                                 pfxB.X := xWinFixToG32(lpAPFX^[i].X);
                                 pfxB.Y := yWinFixToG32(lpAPFX^[i].Y);
                                 if i < pc^.cpfx-2 then
                                  begin
                                       pfxC.X := xWinFixToG32(lpAPFX^[i+1].X);
                                       pfxC.Y := yWinFixToG32(lpAPFX^[i+1].Y);
                                       pfxC.X := (pfxC.X + pfxB.X) div 2;
                                       pfxC.Y := (pfxC.Y + pfxB.Y) div 2;
                                  end else
                                   begin
                                        pfxC.X := xWinFixToG32(lpAPFX^[i+1].X);
                                        pfxC.Y := yWinFixToG32(lpAPFX^[i+1].Y);
                                   end;
                                 BreakSegment(pfxA, pfxB, pfxC);
                                 pfxA := pfxC;
                            end;
                           p2 := pfxC;
                      end;
                  end;
                 ofs2 := ofs2 + SizeOf(TTTPOLYCURVE) + (pc^.cpfx-1)*SizeOf(TPOINTFX);
                 pChar(pc) := pChar(pc) + SizeOf(TTTPOLYCURVE) + (pc^.cpfx-1)*SizeOf(TPOINTFX);
            end;
           if not Done then
            begin
                 p1 := p2;
                 p2 := ps;
                 ofs := ofs + pcSize + SizeOf(TTTPOLYGONHEADER);
                 Done := (ofs >= (xBufSize - SizeOf(TTTPolygonHeader))) ;
                 if (not PointsAreEqual( p1, p2 )) then  GlyphPolygon.Add(p2);
                 pChar(xBufPtr) := pChar(pc);
                 inc( polyN );
        end;
      end;
   { TODO :    }
end;

{ TBitmap32Ex }

constructor TBitmap32Ex.Create;
begin
  inherited Create;

  fCanvas := tCanvas.Create;
  fCanvas.Handle := Self.Handle;

  fDrawOrign.x := 0; fDrawOrign.y := 0;

  fFontMat2 := VertFlip_mat2;

  Font.Size := 24;
  Font.Name := 'Tahoma';
  Font.Color := clBlack;
  Font.Style := [fsBold];
  UpdateFont;
end;

destructor TBitmap32Ex.Destroy;
begin
  fCanvas.Free;
  inherited Destroy;
end;

{ rus:       Font     }
procedure TBitmap32Ex.DrawGlyph(xCharCode : longword; xLeft, yTop: GR32.tFixed;
                                xColor : tColor32; xOptions : tPolygonDrawOptions);
var
   Res, bufSize : LongInt;
   bufPtr, buf  : PTTPolygonHeader;
   dc : hDC;
   gm : TGLYPHMETRICS; // rus:    

{   fUnicode : boolean; }// rus:   
{   i : integer;      }
begin
     { rus: --- I.  glyph-bufer     GetGlyphOutline   fFont }
     UpdateFont;  {       }
     dc := Self.handle; {      }

{     FUNICODE := false;}

{     if not FUNICODE then}  bufSize := GetGlyphOutline( dc ,xCharCode,GGO_NATIVE,gm,0,nil,fFontMat2 )
{                     else  bufSize := GetGlyphOutlineW( dc,xCharCode,GGO_NATIVE,gm,0,nil,fFontMat2 )} ;
     if {(bufSize = GDI_ERROR) or} (bufSize = 0) then exit;

     { rus:  :     gm : TGLYPHMETRICS; //    }

     bufPtr := AllocMem( bufSize );
     buf := bufPtr;
{     if not FUNICODE then}  Res := GetGlyphOutline( dc, xCharCode,GGO_NATIVE,gm,bufSize, pchar(buf),fFontMat2 )
{                     else Res := GetGlyphOutlineW( dc, xCharCode,GGO_NATIVE,gm,bufSize, pchar(buf),fFontMat2 )};
if res <0 then ;
     if {(res = GDI_ERROR) or} (buf^.dwType <> TT_POLYGON_TYPE) then
      begin
           FreeMem( bufPtr );
           Exit;
      end;

    { rus: --- II. c  tArrayOfArrayOfFixed,    ,   }
    BuildGlyphPolygon(bufPtr, {   ,       }
                          bufSize, {   }
                          Bezier2SegmentMinLengthInPixel  shl 16,   {     ( )      }
                          xLeft, yTop,          {    ,        }
                          gm,                             {           . }
                          FixedRect(rect(0, 0, Self.Width, Self.Height)));   {    }

    FreeMem( bufPtr );

   {rus:  --- III.       }
   gPolyPolygon(Self, GlyphPolygon.Points, xColor, xOptions, true);

end;

procedure TBitmap32Ex.RenderTextEx(xLeft, yTop: GR32.tFixed; const xText: string;
                                xColor : tColor32; xOptions : tPolygonDrawOptions);
var
   vShift : GR32.tFixedPoint;

   Res, bufSize : LongInt;
   bufPtr, buf  : PTTPolygonHeader;
   dc : hDC;
   gm1, gm : TGLYPHMETRICS; // rus:    

{   fUnicode : boolean; }// rus:   
   i : integer;
   vCharCode : longword;
begin
  {  --- rus: I.  glyph-bufer     GetGlyphOutline   fFont }
     if abs(detmat2(fFontMat2))<= 0.01 then exit;

     UpdateFont;  {       }
     dc := Self.handle; {      }

     { TODO : rus:      xChar^ Unicode' ,   GetFontUnicodeRanges
                 FUNICODE }

     vShift := FixedPoint(0, 0);
     for i := 1 to length(xText) do
       begin
       vCharCode := ord(xText[i]);

{       FUNICODE := false;}
{       if not FUNICODE then}  bufSize := GetGlyphOutline( dc ,vCharCode,GGO_NATIVE,gm1,0,nil,{fFontMat2} VertFlip_mat2)
{                       else  bufSize := GetGlyphOutlineW( dc,vCharCode,GGO_NATIVE,gm1,0,nil, VertFlip_mat2) };
       if (bufSize = 0) then
        begin
        vShift.x := vShift.x + GR32.Fixed(gm1.gmCellIncX);
        vShift.y := vShift.y - GR32.Fixed(gm1.gmCellIncY);
        continue;
        end;

       { rus:  :     gm : TGLYPHMETRICS; //    }
           bufPtr := AllocMem( bufSize );
         try
{           bufPtr := AllocMem( bufSize );}
           buf := bufPtr;
{           if not FUNICODE then }Res := GetGlyphOutline( dc, vCharCode,GGO_NATIVE,gm,bufSize, pchar(buf),fFontMat2{VertFlip_mat2} )
{                           else Res := GetGlyphOutlineW( dc, vCharCode,GGO_NATIVE,gm,bufSize, pchar(buf),fFontMat2)};
 if res =0 then ;
           if {(res = GDI_ERROR) or} (buf^.dwType <> TT_POLYGON_TYPE) then
            begin
                 bufPtr := nil;
                 FreeMem( bufPtr );
                 continue;
            end;

         { --- rus: II. c  tArrayOfArrayOfFixed,    ,   }
          BuildGlyphPolygon(bufPtr, {   ,       }
                                bufSize, {   }
                                Bezier2SegmentMinLengthInPixel  shl 16,   {     ( )      }
                                xLeft + vShift.x, yTop +  vShift.y,       {    ,        }
                                gm1,                                      {           . }
                                FixedRect(rect(0, 0, Self.Width, Self.Height)));   {    }


         vShift.x := vShift.x + GR32.Fixed(gm1.gmCellIncX);
         vShift.y := vShift.y - GR32.Fixed(gm1.gmCellIncY);
        { --- rus: III.       }
         PolyPolygon(GlyphPolygon.Points, xColor, xOptions, true);
       finally
         bufPtr := nil;
         FreeMem( bufPtr );
       end;

     end;
end;

{     xLength  ,     }
procedure TBitmap32Ex.GetPointAtSegment(xSegment: tArrayOfFixedPoint; xLength : GR32.tFixed; out xPathX, xPathY : GR32.tFixed; out xAngle : double);

var
{$IFDEF TextFitting_DEMO}
 Radius : GR32.tFixed;
{$ELSE}

 iSeg : integer;
 p1, p2, p3, p4 : tFixedPoint;
 vSumLength : GR32.tFixed; { c   }
 vSegLength : GR32.tFixed; {  c }

{$ENDIF}
begin

  if (Length(xSegment) <= 1) or ((Length(xSegment) - 1) mod 3 <> 0)  then
    begin
    raise exception.CreateFmt('G32_Interface.GetPointAtSegment error : Invalidate qubic curve segment points count [%d]; must be four ', [Length(xSegment)]);
    exit;
    end;

{ :    45 
    xAngle := DegToRad(45);
    xPathX := xSegment[0].x + xLength;
    xPathY := xSegment[0].y + xLength;
}

{$IFDEF TextFitting_DEMO}
{      :}
    Radius := GR32.Fixed(100); //  
    xAngle := xLength*6/(2*pi*Radius);
    xPathX := xSegment[0].x + trunc(sin(xAngle)*Radius);
    xPathY := xSegment[0].y - trunc(cos(xAngle)*Radius);
{$ELSE}
  {1.   }
  BuildApproxCurve(xSegment, 4 shl 16, FixedRect(rect(0, 0, Self.Width, Self.Height)));
  {.      CurvePoints : array of TFixedPoint  - CurvePointsCount,      }

  vSumLength := 0;
  {2.        }
  for iSeg := 0 to (CurvePointsCount - 1) div 3 - 1 do
    begin
    {   : }
    p1 := CurvePoints[3*iSeg];
    p2 := CurvePoints[3*iSeg + 1];
    p3 := CurvePoints[3*iSeg + 2];
    p4 := CurvePoints[3*iSeg + 3];
    vSegLength := SegmentLengthValuation(p1, p2, p3, p4);
    if  (vSumLength <= xLength) and (vSumLength + vSegLength >= xLength ) then
      begin
      break;
      end;
    inc(vSumLength, vSegLength);
    end;

  {. iSeg -     (p1, p2, p3, p4) }
  {3.     :}

  { TODO :    xLength - vSumLength }
  xPathX := Mdl(p1.x, p4.x);
  xPathY := Mdl(p1.y, p4.y);
  { TODO :  }
  if (p4.x = p1.x) then
    begin
    if (p4.y  < p1.y) then xAngle := pi/4 else xAngle := - pi/4;
    end
  else
    begin
    xAngle := ArcTan2(p4.y - p1.y, p4.x - p1.x);
    end;

{$ENDIF}
end;


{     xPath}
procedure TBitmap32Ex.RenderFittedText(const xText : string; xColor : tColor32; xOptions : tPolygonDrawOptions;
                               xPath : tArrayOfFixedPoint);
var
   vShift : GR32.tFixedPoint;

   Res, bufSize : LongInt;
   bufPtr, buf  : PTTPolygonHeader;
   dc : hDC;
   gm1, gm : TGLYPHMETRICS; // rus:    

   fUnicode : boolean; // rus:   
   i : integer;
   vCharCode : longword;

   PathX, PathY : GR32.tFixed; {   }
   vTangent : double; {   }

   vFontMat2 : tMat2;
begin
  {,  xPath -   }
  if (Length(xPath) <= 1) or ((Length(xPath) - 1) mod 3 <> 0) then
  begin
    raise exception.CreateFmt('G32_Interface.GetPointAtSegment error : Invalidate qubic curve segment points count [%d]; must be four ', [Length(xPath)]);
    exit;
  end;
  self.PolyBizer(xPath, clGreen32, xOptions, false);


  {  --- rus: I.  glyph-bufer     GetGlyphOutline   fFont }
     UpdateFont;  {       }
     dc := Self.handle; {      }

     { TODO : rus:      xChar^ Unicode' ,   GetFontUnicodeRanges
                 FUNICODE }

     vShift := FixedPoint(0, 0);
     for i := 1 to length(xText) do
       begin
       vCharCode := ord(xText[i]);

       FUNICODE := false;
       if not FUNICODE then  bufSize := GetGlyphOutline( dc ,vCharCode,GGO_NATIVE,gm1,0,nil,{fFontMat2} VertFlip_mat2)
                       else  bufSize := GetGlyphOutlineW( dc,vCharCode,GGO_NATIVE,gm1,0,nil,{fFontMat2} VertFlip_mat2) ;
       if (bufSize = 0) then
        begin
        vShift.x := vShift.x + GR32.Fixed(gm1.gmCellIncX);
        vShift.y := vShift.y - GR32.Fixed(gm1.gmCellIncY);       // == gmCellIncY == 0,  VertFlip_mat2
        continue;
        end;

       {     }
       GetPointAtSegment(xPath, vShift.x, PathX, PathY, vTangent);

       {     fFontMat2,      }
       {       vFontMat2 }
       vFontMat2 := GetRotatedMat2(vTangent);

       vFontMat2 := MultMat2(fFontMat2, vFontMat2);
       if abs(DetMat2(vFontMat2)) > 0.01 then
         begin
           try
             bufPtr := AllocMem( bufSize );
             buf := bufPtr;
{             if not FUNICODE then }Res := GetGlyphOutline( dc, vCharCode,GGO_NATIVE,gm,bufSize, pchar(buf),vFontMat2 )
{                             else Res := GetGlyphOutlineW( dc, vCharCode,GGO_NATIVE,gm,bufSize, pchar(buf),vFontMat2 )};
  if res =0 then ;
             if {(res = GDI_ERROR) or} (res <> bufSize) or (buf^.dwType <> TT_POLYGON_TYPE) then
              begin
                BufPtr := nil; {!!! }
                FreeMem(bufPtr);
                continue;
              end;

           { --- rus: II. c  tArrayOfArrayOfFixed,    ,   }
            BuildGlyphPolygon(bufPtr, {   ,       }
                                  bufSize, {   }
                                  Bezier2SegmentMinLengthInPixel  shl 16,   {     ( )      }
                                  PathX, PathY,       {    ,        }
                                  gm,                                      {           . }
                                  FixedRect(rect(0, 0, Self.Width, Self.Height)));   {    }
                {   }
           vShift.x := vShift.x + GR32.Fixed(gm1.gmCellIncX) + GR32.Fixed(2); { c   }
           vShift.y := vShift.y - GR32.Fixed(gm1.gmCellIncY) - GR32.Fixed(2); {}
          { --- rus: III.       }
           PolyPolygon(GlyphPolygon.Points, xColor, xOptions, true);
         finally
           bufPtr := nil;
           FreeMem( bufPtr );
         end;
       end;
     end;
end;

{      (       )  }
function TBitmap32Ex.SelectFontMat2(xValue : tMat2) : tMat2;
begin
  result := fFontMat2;
  fFontMat2 := xValue;
end;

function TBitmap32Ex.SelectFontTransform(xValue : tFloatMatrix) : tFloatMatrix; {     }
begin
{  result := }
  result[2,0] := fFont_e31;
  result[2,1] := fFont_e32;

  SelectFontMat2(FloatMatrixToMat2(xValue));

  fFont_e31 := xValue[2,0];
  fFont_e31 := xValue[2,1];
end;

function TBitmap32Ex.GetCanvas: tCanvas;
begin
  fCanvas.Handle := Self.Handle;  {!!   ,    }
  result := fCanvas;
end;

procedure TBitmap32Ex.PolyBizer(const Points: TArrayOfFixedPoint;
  Color: TColor32; Options: tPolygonDrawOptions; Closed: Boolean;
  FillMode: TPolyFillMode);
var
  i : integer;
begin
  for i := 0 to Length(Points) - 1 do
    begin
    Points[i].x := Points[i].x - fDrawOrign.x;
    Points[i].y := Points[i].y - fDrawOrign.y;
    end;

  gPolyBezier(Self, Points, Color, Options, Closed, FillMode);
end;

{ rus:   }
procedure TBitmap32Ex.Ellipse(xRect : TFixedRect; xColor: TColor32; xOptions : tPolygonDrawOptions);
const
  EllipseToCurveCoeff = 0.2753905;
var
  PP : TArrayOfFixedPoint;
  dy, dx : integer;
  C      : tFixedPoint; // 

  function point(x, y : GR32.tFixed) : tFixedPoint;
  begin
    result.x := x;
    result.y := y;
  end;
begin
  c := MiddleLine(xRect.TopLeft, xRect.BottomRight);

  dx := trunc((xRect.Right - xRect.Left)*EllipseToCurveCoeff);
  dy := trunc((xRect.Bottom - xRect.Top)*EllipseToCurveCoeff);
  SetLength(PP, 3*4 + 1);

  PP[0] := point(xRect.Left, c.Y);
  PP[1] := point(xRect.Left, c.Y - dy);
  PP[2] := point(c.x - dx, xRect.Top );
  PP[3] := point(c.x, xRect.Top);
  PP[4] := point(c.x + dx, xRect.Top);
  PP[5] := point(xRect.Right, c.Y - dy);
  PP[6] := point(xRect.Right, c.Y);
  PP[7] := point(xRect.Right, c.Y + dy);
  PP[8] := point(c.x + dx, xRect.Bottom );
  PP[9] := point(c.x, xRect.Bottom);
  PP[10] := point(c.x - dx, xRect.Bottom);
  PP[11] := point(xRect.Left, c.Y + dy);
  PP[12] := point(xRect.Left, c.Y);
  Self.PolyBizer(PP, xColor, xOptions, true);
  PP := nil;
end;


procedure TBitmap32Ex.Polygon(const Points: TArrayOfFixedPoint;
  Color: TColor32; Options: tPolygonDrawOptions; Closed: Boolean;
  FillMode: TPolyFillMode);
var
  i : integer;
begin
  for i := 0 to Length(Points) - 1 do
    begin
    Points[i].x := Points[i].x - fDrawOrign.x;
    Points[i].y := Points[i].y - fDrawOrign.y;
    end;
  gPolygon(Self, Points, Color, Options, Closed, FillMode);
end;

procedure TBitmap32Ex.PolyPolygon(
  const Points: TArrayOfArrayOfFixedPoint; Color: tColor32;
  Options: tPolygonDrawOptions; Closed: Boolean; FillMode: TPolyFillMode);
var
  i, j : integer;
  xPoints : tArrayOfFixedPoint;
begin
  for i := 0 to Length(Points) - 1 do
    begin
    setlength(xPoints,i+1);
    xPoints := Points[i];
      for j := 0 to Length(xPoints) - 1 do
        begin
        xPoints[j].x := xPoints[j].x - fDrawOrign.x;
        xPoints[j].y := xPoints[j].y - fDrawOrign.y;
        end;
    end;
  gPolyPolygon(Self, Points, Color, Options, Closed, FillMode);
end;

procedure TBitmap32Ex.PolyPolyBezier(const Points : TArrayOfArrayOfFixedPoint; Color : tColor32; Options : tPolygonDrawOptions;
                     Closed: Boolean; FillMode: TPolyFillMode = pfAlternate);
var
  i, j : integer;
  xPoints : tArrayOfFixedPoint;
begin
  for i := 0 to Length(Points) - 1 do
    begin
    setlength(xPoints,i+1);    
    xPoints := Points[i];
      for j := 0 to Length(xPoints) - 1 do
        begin
        xPoints[j].x := xPoints[j].x - fDrawOrign.x;
        xPoints[j].y := xPoints[j].y - fDrawOrign.y;
        end;
    end;
  gPolyPolyBezier(Self, Points, Color, Options, Closed, FillMode);
end;

initialization

  CP_Clear;
  SetLength(CurvePoints, MaxCurvePointsCount);

  GlyphPolygon := TPolygon32.Create;

finalization

  GlyphPolygon.Free;

end.
