unit InfoJpgFrm;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, JPEG, JPGComment, ExtCtrls;

type
  TJpgInfo = class(TForm)
    ProgressiveDisplay: TCheckBox;
    Scale: TComboBox;
    PixelFormat: TComboBox;
    ColorSpace: TComboBox;
    Performance: TComboBox;
    Label13: TLabel;
    Label12: TLabel;
    Memo1: TMemo;
    IICloseBtn: TBitBtn;
    IIHelpBtn: TBitBtn;
    FileOpenBtn: TSpeedButton;
    OpenDialog1: TOpenDialog;
    JPGComment1: TJPGComment;
    FileSaveBtn: TSpeedButton;
    SaveDialog1: TSaveDialog;
    ScrollBox1: TScrollBox;
    Image1: TImage;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure IIHelpBtnClick(Sender: TObject);
    procedure IICloseBtnClick(Sender: TObject);
    procedure FileOpenBtnClick(Sender: TObject);
    procedure FileSaveBtnClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  JpgInfo: TJpgInfo;

implementation
uses TongGlobals;
{$R *.DFM}

procedure TJpgInfo.FormCreate(Sender: TObject);
begin
  left := JpgInfoFormX;
  top := JpgInfoFormY;
  Scale.ItemIndex := 0; {1:1}
  PixelFormat.ItemIndex := 0;  {24 bit}
  Colorspace.ItemIndex := 0; {RGB}
  Performance.ItemIndex := 0; {Quality}
end;

procedure TJpgInfo.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  JpgInfoFormX := JpgInfo.left;
  JpgInfoFormY := JpgInfo.top;
end;

procedure TJpgInfo.IICloseBtnClick(Sender: TObject);
begin
  Close;
end;
//--------------------------------------------------------
//--------------------------------------------------------
procedure TJpgInfo.IIHelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(1600);
end;
//--------------------------------------------------------
//--------------------------------------------------------
(*           JPGComment1

JPGComment version 0.1 beta

Tested with Delphi 3.0 standard but should run in 2,3,4,5

(C) 1999 by Luke Komsta luke@ariadna.pl

-----------

DESCRIPTION

The JPG files can contain a comments. It is very good thing, but not used by
many JPG authors. It can contain copyright info, description of photo or some
tags which made sorting of pictures much easier. JPGComment is simple
component allows you to read, delete and change comments in JPGs.

PROPERTIES

FileName : string;
         contains a JPG file name

Comment : string;
         contains an actual comment. It of course can contain CR-LF sequences,
         so you can store a memo lines by
         jpgcomment1.comment:=memo1.lines.text or etc.

METHODS

ReadComment;
         reads a comment from file which name is stored in FileName. After
         that, readed comment is stored in Comment propety.

WriteComment;
         writes a comment stored in Comment into a file in FileName. It
         deletes a comment in this file, replacing by new. If Comment is empty
         string, it will only delete all comments in this file.

WriteCommentTo ( nameoffile : string );
         reads a file from FileName, deletes all comments, inserts new
         comment
         if Comment isn't empty, and stores changed JPG in nameoffile.
         Original file is not changed.

KNOWN BUGS:

-coding style is awful ;-) but i had no time to make it more beautiful...

-cannot support multiple comments. JPG standard does not recommend to place
 more comment blocks in one JPG, but sometimes we can have such a file. Then
 JPGComment will get only last comment block by ReadComment and delete all of
 them by write methods.

-if JPG file is not valid, f.e. it is empty file, JPGComment will raise
 exception.

LICENSE TERMS

This software is free. You may use it in all your projects, commercial or not.
Info about my component will be greatly appreciated. It is provided "as is"
and i cannot be sure if it works 100% properly. Please report about any
problems.

Luke Komsta
Po Box 118
24-100 Pulawy-1
Poland

luke@ariadna.pl

http://www.sp8qed.ampr.org/
*)

procedure TJpgInfo.FileOpenBtnClick(Sender: TObject);
begin
  OpenDialog1.Filter:='jpg graphics|*.jpg;*.jpeg';
  OpenDialog1.InitialDir:=ImagePath;
  OpenDialog1.FileName:='*.jpg';//Changefileext(TempName,'.jpeg' );
  if OpenDialog1.Execute then
  begin
    memo1.Clear;
    ImagePath:=ExtractFilePath(OpenDialog1.FileName);
    JPGComment1.FileName:=OpenDialog1.FileName;
    JPGComment1.ReadComment;
    memo1.lines.text:=jpgcomment1.comment;
    {GCForm.DoLoad(OpenDialog1.FileName); }
    Image1.Picture.LoadFromFile(OpenDialog1.FileName);
    with TJPEGImage(Image1.Picture.Graphic) do
    begin
      PixelFormat := TJPEGPixelFormat(Self.PixelFormat.ItemIndex);
      Scale := TJPEGScale(Self.Scale.ItemIndex);
      Grayscale := Boolean(Self.Colorspace.ItemIndex);
      Performance := TJPEGPerformance(Self.Performance.ItemIndex);
    end;
    Scale.Enabled := True;
    PixelFormat.Enabled := True;
    Colorspace.Enabled := True;
    Performance.Enabled := True;
    ProgressiveDisplay.Enabled :=
        TJPEGImage(Image1.Picture.Graphic).ProgressiveEncoding;
    ProgressiveDisplay.Checked:=
        TJPEGImage(Image1.Picture.Graphic).ProgressiveEncoding;
    {GCForm.Image1.IncrementalDisplay := IncrementalDisplay.Checked;}
  end;
end;

procedure TJpgInfo.FileSaveBtnClick(Sender: TObject);
begin
  SaveDialog1.Filter:='jpg graphics|*.jpg;*.jpeg';
  SaveDialog1.InitialDir:=ImagePath;
  SaveDialog1.FileName:=OpenDialog1.FileName;
  if SaveDialog1.Execute then
  begin
      jpgcomment1.comment:=memo1.lines.text;
      JPGComment1.FileName:=SaveDialog1.FileName;
      jpgcomment1.writecomment;
  end;
end;

end.
