unit PanZoomRPaintFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, PNGImage,
  Controls, Forms, Dialogs,
  PanZoom, StdCtrls, ExtCtrls, ComCtrls, Buttons;

type
  TPanZoomRPaintForm = class(TForm)
    Panel1: TPanel;
    cmdZoomIn: TButton;
    cmdOneToOne: TButton;
    cmdZoomOut: TButton;
    Splitter1: TSplitter;
    cmdSizeToImage: TButton;
    cmdPaint: TButton;
    lblXY: TLabel;
    lblStatus: TLabel;
    chkProportionalZoom: TCheckBox;
    chkAligned: TCheckBox;
    cmdLoadImage: TButton;
    OpenDialog1: TOpenDialog;
    chkResizable: TCheckBox;
    lblPenWidth: TLabel;
    txtPenWidth: TEdit;
    udPenWidth: TUpDown;
    ColorDialog1: TColorDialog;
    ColorPanel: TPanel;
    HelpBtn: TSpeedButton;
    ExitBtn: TSpeedButton;
    SaveBtn: TSpeedButton;
    SaveDialog1: TSaveDialog;
    PanZoomPanel1: TPanZoomPanel;
    procedure cmdZoomInClick(Sender: TObject);
    procedure cmdOneToOneClick(Sender: TObject);
    procedure cmdZoomOutClick(Sender: TObject);
    procedure cmdSizeToImageClick(Sender: TObject);
    procedure cmdPaintClick(Sender: TObject);
    procedure chkAlignedClick(Sender: TObject);
    procedure PanZoomPanel1MouseMove(Sender: TObject; Shift: TShiftState;
                                     DeviceX, DeviceY: Integer;
                                     ActualX, ActualY: Double);
    procedure PanZoomPanel1ZoomInBegin(Sender: TObject);
    procedure PanZoomPanel1PaintBegin(Sender: TObject);
    procedure cmdLoadImageClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormCreate(Sender: TObject);
    procedure chkResizableClick(Sender: TObject);
    procedure txtPenWidthChange(Sender: TObject);
    procedure ColorPanelClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure ExitBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
  private
    procedure CheckWidgets;
    procedure SetPenWidth;
  public
  end;

var
  PanZoomRPaintForm: TPanZoomRPaintForm;

implementation
uses TongGlobals;
{$R *.DFM}

procedure TPanZoomRPaintForm.FormCreate(Sender: TObject);
begin
  left := PanZoomRPaintFormX;
  top := PanZoomRPaintFormY;
  CheckWidgets;
end;

procedure TPanZoomRPaintForm.ExitBtnClick(Sender: TObject);
begin
  Close;
end;
procedure TPanZoomRPaintForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  PanZoomRPaintFormX := PanZoomRPaintForm.left;
  PanZoomRPaintFormY := PanZoomRPaintForm.top;
end;

procedure TPanZoomRPaintForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(550);
end;


procedure TPanZoomRPaintForm.cmdZoomInClick(Sender: TObject);
begin
  PanZoomPanel1.ProportionalZoom := chkProportionalZoom.Checked;
  PanZoomPanel1.ZoomingIn := not PanZoomPanel1.ZoomingIn;
end;

procedure TPanZoomRPaintForm.cmdOneToOneClick(Sender: TObject);
begin
  PanZoomPanel1.OneToOne;
end;

procedure TPanZoomRPaintForm.cmdZoomOutClick(Sender: TObject);
var
  Factor: string;
begin
  Factor := '1.0';
  if InputQuery('Zoom Out', 'Enter factor:', Factor) then begin
    PanZoomPanel1.ZoomOut(StrToFloat(Factor));
  end;
end;

procedure TPanZoomRPaintForm.cmdSizeToImageClick(Sender: TObject);
begin
  PanZoomPanel1.SizeToImage;
end;

procedure TPanZoomRPaintForm.cmdPaintClick(Sender: TObject);
begin
  PanZoomPanel1.Painting := not PanZoomPanel1.Painting;
end;

procedure TPanZoomRPaintForm.chkAlignedClick(Sender: TObject);
const
  Aligns: array [Boolean] of TAlign = (alNone, alClient);
begin
  if chkAligned.Checked then chkResizable.Checked := False;
  PanZoomPanel1.Align := Aligns[chkAligned.Checked];
end;

procedure TPanZoomRPaintForm.PanZoomPanel1MouseMove(Sender: TObject;
                                        Shift: TShiftState;
                                        DeviceX, DeviceY: Integer;
                                        ActualX, ActualY: Double);
const
  sFormat = '0.00';
begin
  if not PanZoomPanel1.ScaleDefined then Exit;
  lblXY.Caption := '(' + FormatFloat(sFormat, ActualX) + ', ' +
                   FormatFloat(sFormat, ActualY) + ')';
end;

procedure TPanZoomRPaintForm.PanZoomPanel1ZoomInBegin(Sender: TObject);
const
  ZoomingStatus: array [Boolean] of string = ('', 'Zooming');
  ZoomInCaptions: array [Boolean] of string = ('Zoom &In', '&Cancel');
begin
  lblStatus.Caption := ZoomingStatus[PanZoomPanel1.ZoomingIn];
  cmdZoomIn.Caption := ZoomInCaptions[PanZoomPanel1.ZoomingIn];
  CheckWidgets;
end;

procedure TPanZoomRPaintForm.PanZoomPanel1PaintBegin(Sender: TObject);
const
  PaintingStatus: array [Boolean] of string = ('', 'Painting');
  PaintCaptions: array [Boolean] of string = ('&Paint', '&Cancel');
begin
  lblStatus.Caption := PaintingStatus[PanZoomPanel1.Painting];
  cmdPaint.Caption := PaintCaptions[PanZoomPanel1.Painting];
  CheckWidgets;
end;

procedure TPanZoomRPaintForm.CheckWidgets;
begin
  {Dunno about this Load  1 only}
  cmdLoadImage.Enabled := not Assigned(PanZoomPanel1.Bitmap);
  SaveBtn.Enabled := Assigned(PanZoomPanel1.Bitmap);
  cmdZoomIn.Enabled := Assigned(PanZoomPanel1.Bitmap) and
                       not PanZoomPanel1.Painting;
  cmdOneToOne.Enabled := Assigned(PanZoomPanel1.Bitmap) and
                         not PanZoomPanel1.ZoomingIn and
                         not PanZoomPanel1.Painting;
  cmdZoomOut.Enabled := Assigned(PanZoomPanel1.Bitmap) and
                        not PanZoomPanel1.ZoomingIn and
                        not PanZoomPanel1.Painting;
  cmdSizeToImage.Enabled := Assigned(PanZoomPanel1.Bitmap) and
                            not PanZoomPanel1.ZoomingIn and
                            not PanZoomPanel1.Painting;
  cmdPaint.Enabled := Assigned(PanZoomPanel1.Bitmap) and
                      not PanZoomPanel1.ZoomingIn;
end;

procedure TPanZoomRPaintForm.cmdLoadImageClick(Sender: TObject);
var
  PNG:TPNGObject;
  Bitmap: TBitmap;
begin
    OpenDialog1.InitialDir := ImagePath;
    OpenDialog1.Filter := 'All (*.bmp;*.png)|*.bmp;*.png';
  if OpenDialog1.Execute then
  begin
    Bitmap := TBitmap.Create;
    try
  if (lowercase(extractfileext(OpenDialog1.FileName)) = '.bmp') then
  begin
      Bitmap.LoadFromFile(OpenDialog1.FileName);
  end else
  if (lowercase(extractfileext(OpenDialog1.FileName)) = '.png') then
  begin
    PNG := TPNGObject.Create;
    {In case something goes wrong, free both PNG and Bitmap}
    try
      PNG.LoadFromFile(OpenDialog1.FileName);
      Bitmap.Assign(PNG); //Convert data into bitmap
    finally
      PNG.Free;
    end;
  end;
      PanZoomPanel1.Bitmap := Bitmap;
      PanZoomPanel1.SetFirstScalePoint(MakeLongintPoint(0, Bitmap.Height),
                                       MakeDoublePoint(0, 0));
      PanZoomPanel1.SetSecondScalePoint(MakeLongintPoint(Bitmap.Width, 0),
                                        MakeDoublePoint(100, 100));
      PanZoomPanel1.DefineScaleNoSkew(False, False);
    finally
      Bitmap.Free;
    end;
  end;
  CheckWidgets;
end;

procedure TPanZoomRPaintForm.SaveBtnClick(Sender: TObject);
var
  Bitmap: TBitmap;
begin
    SaveDialog1.filter := 'BMP files|*.bmp';
    SaveDialog1.InitialDir := ImagePath;
    SaveDialog1.DefaultExt := 'bmp';
  if SaveDialog1.Execute then
  begin
    Bitmap := TBitmap.Create;
      Bitmap.Assign(PanZoomPanel1.Bitmap);
    try
      Bitmap.SaveToFile(ChangeFileExt(SaveDialog1.FileName, '.bmp'));
    finally
      Bitmap.Free;
    end;
  end;
end;

procedure TPanZoomRPaintForm.chkResizableClick(Sender: TObject);
begin
  PanZoomPanel1.Sizable := chkResizable.Checked;
end;

procedure TPanZoomRPaintForm.SetPenWidth;
begin
  PanZoomPanel1.PenWidth := StrToIntDef(txtPenWidth.Text, 1);
end;

procedure TPanZoomRPaintForm.txtPenWidthChange(Sender: TObject);
begin
  SetPenWidth;
end;
{procedure SetPen(const ACanvas: TCanvas; const AColor: TColor;
                 const AMode: TPenMode; const AStyle: TPenStyle;
                 const AWidth: Integer);
procedure SetBrush(const ACanvas: TCanvas; const AColor: TColor;
                   const AStyle: TBrushStyle);
procedure SetPenAndBrush(const ACanvas: TCanvas; const APenColor: TColor;
                         const AMode: TPenMode; const APenStyle: TPenStyle;
                         const AWidth: Integer; const ABrushColor: TColor;
                         const ABrushStyle: TBrushStyle);

TPenStyle = (psSolid, psDash, psDot, psDashDot, psDashDotDot, psClear, psInsideFrame);
 TPenMode = (pmBlack, pmWhite, pmNop, pmNot, pmCopy, pmNotCopy, pmMergePenNot, pmMaskPenNot, pmMergeNotPen, pmMaskNotPen, pmMerge, pmNotMerge, pmMask, pmNotMask, pmXor, pmNotXor)
}
procedure TPanZoomRPaintForm.ColorPanelClick(Sender: TObject);
begin
  ColorDialog1.Color:=ColorPanel.Color;
  if ColorDialog1.Execute then
begin
  ColorPanel.Color := ColorDialog1.Color;
PanZoomPanel1.SetPen(
PanZoomPanel1.Bitmap.Canvas,
ColorPanel.Color,
pmCopy,
psSolid,
PanZoomPanel1.PenWidth );
  {with PanZoomPanel1.Canvas.Pen do begin
    Color := ColorPanel.Color;
    Mode := pmCopy;
    Style := psSolid;
    Width := PanZoomPanel1.PenWidth;
  end; }
  end;
end;





end.
