// See also, Borland's TI 3332, "Converting a BMP to a JPEG and Vice Versa"
// www.borland.com/devsupport/dephi/ti_list/TI3332.html

unit ScreenBMPJPG;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, JPEG, ExtDlgs, Spin, ComCtrls;

type
  TFormConvert = class(TForm)
    ButtonPrintJPG: TButton;
    ButtonPrintBMP: TButton;
    LabelBMP: TLabel;
    LabelJPG: TLabel;
    EditBMP: TEdit;
    ButtonBrowseBMP: TButton;
    EditJPG: TEdit;
    ButtonBrowseJPG: TButton;
    SpeedButtonJPGtoBMP: TSpeedButton;
    SpeedButtonBMPtoJPG: TSpeedButton;
    LabelBMPInfo: TLabel;
    LabelJPGInfo: TLabel;
    OpenPictureDialogBMP: TOpenPictureDialog;
    OpenPictureDialogJPG: TOpenPictureDialog;
    ButtonSaveBMP: TButton;
    ButtonSaveJPG: TButton;
    SpinEditQuality: TSpinEdit;
    LabelQuality: TLabel;
    GroupBoxDecompression: TGroupBox;
    LabelPerformance: TLabel;
    ComboBoxPerformance: TComboBox;
    LabelScale: TLabel;
    ComboBoxScale: TComboBox;
    ComboBoxPixelFormat: TComboBox;
    LabelFormat: TLabel;
    CheckBoxProgressiveDisplay: TCheckBox;
    CheckBoxSmoothing: TCheckBox;
    GroupBoxCompressionDecompression: TGroupBox;
    CheckBoxGrayScale: TCheckBox;
    CheckBoxProgressiveEncoding: TCheckBox;
    CheckBoxIncrementalDisplay: TCheckBox;
    CheckBoxStretchBMP: TCheckBox;
    CheckBoxStretchJPG: TCheckBox;
    LabelBMPBytes: TLabel;
    LabelJPGBytes: TLabel;
    LabelCompressionFactor: TLabel;
    LabelCompressionFactorValue: TLabel;
    Label1: TLabel;
    Label2: TLabel;
    LabelProgress: TLabel;
    ProgressBarJPG: TProgressBar;
    ScrollBox1: TScrollBox;
    ImageBMP: TImage;
    ScrollBox2: TScrollBox;
    ImageJPG: TImage;
    SpeedButton1: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure ButtonPrintBMPClick(Sender: TObject);
    procedure ButtonPrintJPGClick(Sender: TObject);
    procedure ButtonBrowseBMPClick(Sender: TObject);
    procedure ButtonBrowseJPGClick(Sender: TObject);
    procedure SpeedButtonBMPtoJPGClick(Sender: TObject);
    procedure ButtonSaveBMPClick(Sender: TObject);
    procedure ButtonSaveJPGClick(Sender: TObject);
    procedure ChangeJPEGInfo(Sender: TObject);
    procedure CheckBoxStretchBMPClick(Sender: TObject);
    procedure CheckBoxStretchJPGClick(Sender: TObject);
    procedure ImageJPGProgressUpdate(Sender: TObject; Stage: TProgressStage;
      PercentDone: Byte; RedrawNow: Boolean; const R: TRect;
      const Msg: String);
    procedure SpeedButtonJPGtoBMPClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  private
    Bitmap       :  TBitmap;     // in-memory version of what's on screen
    JPEGImage    :  TJPEGImage;  // in-memory version of what's on screen

    BMPBytes     :  INTEGER;
    JPGBytes     :  INTEGER;
    StartTick    :  DWORD;      // D3/D4 compatibility

    PROCEDURE LoadBitmap(CONST Filename:  STRING);
    PROCEDURE LoadJPEG(CONST Filename:  STRING);
    PROCEDURE UpdateJPEGInfo;

  public

  end;

var
  FormConvert: TFormConvert;

implementation
{$R *.DFM}

  USES
    TongGlobals,
    PaletteLibrary,    // IsPaletteDevice, CreateOptimizedPaletteForSingleBitmap
    Printers;

  CONST
    PrintedImageWidth = 6.00;   // Inches


//// Local /////////////////////////////////////////////////


FUNCTION GetFileSize(CONST Filename:  STRING):  INTEGER;
  VAR
    FileStream:  TFileStream;
BEGIN
  TRY
    FileStream := TFileStream.Create(Filename, fmOpenRead + fmShareDenyNone);
    RESULT := FileStream.Size;
    FileStream.Free;
  EXCEPT
    RESULT := 0
  END
END {GetFileSize};


FUNCTION GetPixelFormatString(CONST PixelFormat:  TPixelFormat):  STRING;
  VAR
    Format:  STRING;
BEGIN
  CASE PixelFormat OF
    pfDevice:  Format := 'Device';
    pf1bit:    Format := '1 bit';
    pf4bit:    Format := '4 bit';
    pf8bit:    Format := '8 bit';
    pf15bit:   Format := '15 bit';
    pf16bit:   Format := '16 bit';
    pf24bit:   Format := '24 bit';
    pf32bit:   Format := '32 bit'
    ELSE
      Format := 'Unknown';
  END;
  RESULT := Format;
END {GetPixelFormatString};


FUNCTION GetBitmapDimensionsString(CONST Bitmap:  TBitmap):  STRING;
BEGIN
  RESULT := IntToStr(Bitmap.Width)  + ' by ' +
            IntToStr(Bitmap.Height) + ' pixels by ' +
            GetPixelFormatString(Bitmap.PixelFormat) + ' color';
END {GetBitmapDimensionsString};


// This is the recommended way to print images on a printer.
// Based on posting to borland.public.delphi.winapi by Rodney E Geraghty, 8/8/97.}
PROCEDURE PrintBitmap(Canvas:  TCanvas; DestRect:  TRect;  Bitmap:  TBitmap);
  VAR
    BitmapHeader:  pBitmapInfo;
    BitmapImage :  POINTER;
    HeaderSize  :  DWORD;     // D3/D4 compatibility
    ImageSize   :  DWORD;
BEGIN
  GetDIBSizes(Bitmap.Handle, HeaderSize, ImageSize);
  GetMem(BitmapHeader, HeaderSize);
  GetMem(BitmapImage,  ImageSize);
  TRY
    GetDIB(Bitmap.Handle, Bitmap.Palette, BitmapHeader^, BitmapImage^);
    StretchDIBits(Canvas.Handle,
                  DestRect.Left, DestRect.Top,     {Destination Origin}
                  DestRect.Right  - DestRect.Left, {Destination Width}
                  DestRect.Bottom - DestRect.Top,  {Destination Height}
                  0, 0,                            {Source Origin}
                  Bitmap.Width, Bitmap.Height,     {Source Width & Height}
                  BitmapImage,
                  TBitmapInfo(BitmapHeader^),
                  DIB_RGB_COLORS,
                  SRCCOPY)
  FINALLY
    FreeMem(BitmapHeader);
    FreeMem(BitmapImage)
  END
END {PrintBitmap};


//// Form   ////////////////////////////////////////////////
procedure TFormConvert.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Bitmap.Free;
  JPEGImage.Free;
  FormConvertX := FormConvert.left;
  FormConvertY := FormConvert.top;
end;
procedure TFormConvert.SpeedButton1Click(Sender: TObject);
begin
  Application.HelpContext(7555);
end;
procedure TFormConvert.FormCreate(Sender: TObject);
begin
  left := FormConvertX;
  top := FormConvertY;
  {left := 11;
  top := 11;}
  OpenPictureDialogBMP.InitialDir :=ImagePath;// ExtractFileDir(ParamStr(0));
  OpenPictureDialogJPG.InitialDir :=ImagePath;// ExtractFileDir(ParamStr(0));

  Bitmap    := TBitmap.Create;
  ImageBMP.Picture.Graphic := Bitmap;

  JPEGImage := TJPEGImage.Create;
  // Do this to prevent Access Violation -- sure would be nice if parallel
  // constructs worked for TBitmaps and TJPEGImages, but that's asking too much.
  JPEGImage.Assign(ImageBMP.Picture.Graphic);
  ImageJPG.Picture.Graphic := JPEGImage;

  ComboBoxPerformance.ItemIndex := 0;
  ComboBoxScale.ItemIndex       := 0;
  ComboBoxPixelFormat.ItemIndex := 0;

  JPGBytes := 0;
  BMPBytes := 0;

  ImageJPG.OnProgress := ImageJPGProgressUpdate
end;


//// BMP  //////////////////////////////////////////////////

procedure TFormConvert.ButtonBrowseBMPClick(Sender: TObject);
begin
  {OpenPictureDialogBMP.Filter := 'All (*.bmp)|*.bmp';}
  IF   OpenPictureDialogBMP.Execute
  THEN BEGIN
    Screen.Cursor := crHourGlass;
    TRY
      EditBMP.Text := OpenPictureDialogBMP.Filename;
      LabelCompressionFactorValue.Caption := '';
      LoadBitmap(OpenPictureDialogBMP.Filename)
    FINALLY
      Screen.Cursor := crDefault
    END;

    // Start here for next image
    OpenPictureDialogBMP.InitialDir :=
      ExtractFilePath(OpenPictureDialogBMP.Filename);
  END
end;


procedure TFormConvert.ButtonSaveBMPClick(Sender: TObject);
begin
  If FileExists(EditBMP.Text) then
  ImageBMP.Picture.Bitmap.SaveToFile(EditBMP.Text);
  {ShowMessage('File "' + EditBMP.Text + '" saved.')}
end;


procedure TFormConvert.CheckBoxStretchBMPClick(Sender: TObject);
begin
  ImageBMP.Stretch := CheckBoxStretchBMP.Checked
end;


PROCEDURE TFormConvert.LoadBitmap(CONST Filename:  STRING);
BEGIN
  BMPBytes := GetFileSize(Filename);
  LabelBMPBytes.Caption := FormatFloat('#,', BMPBytes) + ' bytes';

  // Get rid of old one and get new bitmap
  Bitmap.Free;
  Bitmap := TBitmap.Create;

  Bitmap.LoadFromFile(Filename);
  ImageBMP.Picture.Graphic := Bitmap;

  ButtonPrintBMP.Enabled := TRUE;
  SpeedButtonBMPtoJPG.Enabled := TRUE;
  LabelBMPInfo.Caption := GetBitmapDimensionsString(TBitmap(ImageBMP.Picture.Graphic));
  SpinEditQuality.Enabled := TRUE
END {LoadBitmap};


procedure TFormConvert.ButtonPrintBMPClick(Sender: TObject);
  VAR
    AspectRatio    :  DOUBLE;
    ImageBottom    :  INTEGER;
    ImageLeft      :  INTEGER;
    ImageRight     :  INTEGER;
    ImageTop       :  INTEGER;
    iPixelsPerInch :  INTEGER;
    jPixelsPerInch :  INTEGER;
    TargetRectangle:  TRect;
begin
  iPixelsPerInch := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
  jPixelsPerInch := GetDeviceCaps(Printer.Handle, LOGPIXELSY);

  Printer.Orientation := poPortrait;
  Printer.BeginDoc;
  AspectRatio := ImageBMP.Picture.Bitmap.Width /
                 ImageBMP.Picture.Bitmap.Height;

  // Hardwire position on page for this demo
  ImageLeft   := ROUND((8.0 - PrintedImageWidth)/2.0 * iPixelsPerInch);
  ImageRight  := ImageLeft + ROUND(PrintedImageWidth*iPixelsPerInch);
  ImageTop    := ROUND(1.50*jPixelsPerInch);
  ImageBottom := ImageTop + ROUND(PrintedImageWidth / AspectRatio * jPixelsPerInch);
  TargetRectangle := Rect(ImageLeft,  ImageTop,
                          ImageRight, ImageBottom);

  Printer.Canvas.TextOut((ImageLeft + ImageRight -
                          Printer.Canvas.TextWidth(EditBMP.Text)) DIV 2,
                         ROUND(1.50*jPixelsPerInch) -  // above the image
                              3*Printer.Canvas.TextHeight(EditBMP.Text) DIV 2,
                         EditBMP.Text);
  Printer.Canvas.TextOut(ImageLeft, ImageBottom, LabelBMPInfo.Caption);
  Printer.Canvas.TextOut(ImageLeft, ImageBottom + Printer.Canvas.TextHeight('X'),
                         LabelBMPBytes.Caption);

  PrintBitmap(Printer.Canvas, TargetRectangle, ImageBMP.Picture.Bitmap);

  Printer.EndDoc;
end;


//// JPG  //////////////////////////////////////////////////

procedure TFormConvert.ButtonBrowseJPGClick(Sender: TObject);
begin
  {OpenPictureDialogJPG.Filter := 'All (*.jpg)|*.jpg'; }
  IF   OpenPictureDialogJPG.Execute
  THEN BEGIN
     EditJPG.Text := OpenPictureDialogJPG.Filename;
     LabelCompressionFactorValue.Caption := '';
     LoadJPEG(OpenPictureDialogJPG.Filename);
     UpdateJPEGInfo;
  END;
end;


procedure TFormConvert.ButtonSaveJPGClick(Sender: TObject);
begin
  If FileExists(EditJPG.Text) then
  TJPEGImage(ImageJPG.Picture.Graphic).SaveToFile(EditJPG.Text);
  {ShowMessage('File "' + EditJPG.Text + '" saved.')}
end;


procedure TFormConvert.CheckBoxStretchJPGClick(Sender: TObject);
begin
  ImageJPG.Stretch := CheckBoxStretchJPG.Checked
end;


PROCEDURE TFormConvert.LoadJPEG(CONST Filename:  STRING);
BEGIN
  JPGBytes := GetFileSize(Filename);
  LabelJPGBytes.Caption := FormatFloat('#,', JPGBytes) + ' bytes';

  // Get rid of old and create new
  JPEGImage.Free;
  JPEGImage := TJPEGImage.Create;

  JPEGImage.LoadFromFile(EditJPG.Text);

  // Don't allow re-save of image just loaded
  ButtonSaveJPG.Enabled  := FALSE;

  // Allow printing now
  ButtonPrintJPG.Enabled := TRUE;

  LabelProgress.Caption := '';
  UpdateJPEGInfo;

  ImageJPG.Picture.Graphic := JPEGImage;

  ButtonPrintJPG.Enabled := TRUE;
  SpeedbuttonJPGtoBMP.Enabled := TRUE
END {LoadJPEG};


PROCEDURE TFormConvert.UpdateJPEGInfo;
  VAR
    EnabledFlag:  BOOLEAN;
BEGIN
  EnabledFlag := ImageJPG.Picture.Graphic is TJPEGImage;

  IF   EnabledFlag
  THEN BEGIN
    WITH TJPEGImage(ImageJPG.Picture.Graphic) DO
    BEGIN
      // Items in ComboBoxPixelFormat must be in same order as TJPEGPixelFormat enumeration
      PixelFormat := TJPEGPixelFormat(Self.ComboBoxPixelFormat.ItemIndex);

      // Items in ComboBoxScale must be in same order as TJPEGScale enumeration
      Scale := TJPEGScale(Self.ComboBoxScale.ItemIndex);

      GrayScale := CheckBoxGrayScale.Checked;

      // Items in ComboBoxPeformance must be in the same order as TJPEGPeformance enumeration
      Performance := TJPEGPerformance(Self.ComboBoxPerformance.ItemIndex);

      ProgressiveDisplay := Self.CheckBoxProgressiveDisplay.Checked;

      Smoothing := Self.CheckBoxSmoothing.Checked;

      LabelJPGInfo.Caption := IntToStr(JPEGImage.Width) + ' by ' + IntToStr(JPEGImage.Height) +
                              ' pixels by ' + Self.ComboBoxPixelFormat.Text + ' color'

    END
  END;

  ComboBoxScale.Enabled              := EnabledFlag;
  ComboBoxPixelFormat.Enabled        := EnabledFlag;
  CheckBoxGrayScale.Enabled          := EnabledFlag;
  ComboBoxPerformance.Enabled        := EnabledFlag;

  CheckBoxProgressiveDisplay.Enabled := EnabledFlag;


  CheckBoxProgressiveDisplay.Enabled := EnabledFlag AND
                                        TJPEGImage(ImageJPG.Picture.Graphic).ProgressiveEncoding;
  ImageJPG.IncrementalDisplay := CheckBoxIncrementalDisplay.Checked

END {UpdateJPGInfo};


procedure TFormConvert.ChangeJPEGInfo(Sender: TObject);
begin
  UpdateJPEGInfo
end;


procedure TFormConvert.ButtonPrintJPGClick(Sender: TObject);
  VAR
    AspectRatio    :  DOUBLE;
    Bitmap         :  TBitmap;
    ImageBottom    :  INTEGER;
    ImageLeft      :  INTEGER;
    ImageRight     :  INTEGER;
    ImageTop       :  INTEGER;
    iPixelsPerInch :  INTEGER;
    jPixelsPerInch :  INTEGER;
    TargetRectangle:  TRect;
    s              :  STRING;
begin
  iPixelsPerInch := GetDeviceCaps(Printer.Handle, LOGPIXELSX);
  jPixelsPerInch := GetDeviceCaps(Printer.Handle, LOGPIXELSY);

  Printer.Orientation := poPortrait;
  Printer.BeginDoc;

  // Maintain JPEG's aspect ratio in printed images, whether stretched or not
  // on the display.
  AspectRatio := TJPEGImage(ImageJPG.Picture.Graphic).Width /
                 TJPEGImage(ImageJPG.Picture.Graphic).Height;

  ImageLeft   := ROUND((8.0 - PrintedImageWidth)/2.0 * iPixelsPerInch);
  ImageRight  := ImageLeft + ROUND(PrintedImageWidth*iPixelsPerInch);
  ImageTop    := ROUND(1.50*jPixelsPerInch);
  ImageBottom := ImageTop + ROUND(PrintedImageWidth / AspectRatio * jPixelsPerInch);
  TargetRectangle := Rect(ImageLeft,  ImageTop,
                          ImageRight, ImageBottom);

  Printer.Canvas.TextOut((ImageLeft + ImageRight -
                          Printer.Canvas.TextWidth(EditBMP.Text)) DIV 2,
                         ROUND(1.50*jPixelsPerInch) -  // above the image
                              3*Printer.Canvas.TextHeight(EditBMP.Text) DIV 2,
                         EditJPG.Text);

  Printer.Canvas.TextOut(ImageLeft, ImageBottom, LabelJPGInfo.Caption);
  s := LabelJPGBytes.Caption;
  IF   LabelCompressionFactorValue.Visible
  THEN s := s + ' (Compression Factor = ' + LabelCompressionFactorValue.Caption + ' : 1)';

  Printer.Canvas.TextOut(ImageLeft, ImageBottom + Printer.Canvas.TextHeight('X'),
                         s);

  Bitmap := TBitmap.Create;
  TRY
    // Convert JPG to Bitmap for printing
    WITH TJPEGImage(ImageJPG.Picture.Graphic)
    DO BEGIN
      Bitmap.Width  := Width;
      Bitmap.Height := Height;

      IF   PixelFormat = jf24Bit     // Must be jf24Bit or jf8Bit
      THEN Bitmap.PixelFormat := pf24Bit
      ELSE Bitmap.PixelFormat := pf8Bit;
      DIBNeeded
    END;

    // According to Borland's TI 3332 ...
    Bitmap.Canvas.Draw(0,0, ImageJPG.Picture.Graphic);

    PrintBitmap(Printer.Canvas, TargetRectangle, Bitmap);
  FINALLY
    Bitmap.Free
  END;
  Printer.EndDoc
end;


procedure TFormConvert.ImageJPGProgressUpdate(Sender: TObject; Stage: TProgressStage;
  PercentDone: Byte; RedrawNow: Boolean; const R: TRect;
  const Msg: String);
begin
  CASE Stage OF
    psStarting:
      BEGIN
        ProgressBarJPG.Position := 0;
        ProgressBarJPG.Visible := TRUE;
        StartTick := GetTickCount;
      END;

    psRunning:
      BEGIN
        ProgressBarJPG.Position := PercentDone;
        LabelProgress.Caption := 'Run ' + Msg;
        LabelProgress.Update
      END;

    psEnding:
      BEGIN
        ProgressBarJPG.Visible := FALSE;
        LabelProgress.Caption :=  'Time = ' +
          Format('%.1f', [(GetTickCount - StartTick)/1000]) +
          ' seconds'
      END
  END;

end;

  
//// Conversions  //////////////////////////////////////////

procedure TFormConvert.SpeedButtonBMPtoJPGClick(Sender: TObject);
  CONST
    TemporaryJPGName = '##TMPJPG##.JPG';

  VAR
    FileName    :  STRING;
    FilePath    :  STRING;
begin
  Screen.Cursor := crHourGlass;
  TRY
    // Create new JPEG object
    JPEGImage.Free;
    JPEGImage := TJPEGImage.Create;

    // Trap any problem with blank Spin Edit box
    TRY
      JPEGImage.CompressionQuality := SpinEditQuality.Value;
    EXCEPT
      SpinEditQuality.Value := 90;
      JPEGImage.CompressionQuality := SpinEditQuality.Value
    END;

    // Set options that affect compression
    JPEGImage.ProgressiveEncoding := CheckBoxProgressiveEncoding.Checked;
    JPEGImage.Grayscale           := CheckBoxGrayScale.Checked;

    // Convert BMP to JPEG
    JPEGImage.Assign(Bitmap);

    JPEGImage.SaveToFile(TemporaryJPGName);
    JPGBytes := GetFileSize(TemporaryJPGName);
    LabelJPGBytes.Caption := FormatFloat('#,', JPGBytes) + ' bytes';

    IF   (JPGBytes > 0) AND (BMPBytes > 0)
    THEN BEGIN
      LabelCompressionFactorValue.Caption :=
        Format('%5.1f', [BMPBytes / JPGBytes]);

      LabelCompressionFactorValue.Visible := TRUE;
    END
    ELSE LabelCompressionFactorValue.Visible := FALSE;

    ImageJPG.Picture.LoadFromFile(TemporaryJPGName);
    DeleteFile(TemporaryJPGName);

    // Make JPEG filename to match BMP but with Quality Factor and .JPG
    FileName := ExtractFileName(EditBMP.Text);
    FilePath := ExtractFilePath(EditBMP.Text);
    FileName := ChangeFileExt(FileName, '.JPG');

    FileName := FilePath + COPY(FileName,1,LENGTH(FileName)-4) +
                Format('%3.3d', [SpinEditQuality.Value]) +
                COPY(FileName,LENGTH(FileName)-4+1, 4);
    EditJPG.Text := FileName;

    ButtonSaveJPG.Enabled := TRUE;
    ButtonPrintJPG.Enabled := TRUE;
    ButtonPrintJPG.Visible := TRUE;

    UpdateJPEGInfo
  FINALLY
    Screen.Cursor := crDefault
  END
end;


procedure TFormConvert.SpeedButtonJPGtoBMPClick(Sender: TObject);
  VAR
    FileName    :  STRING;
    MemoryStream:  TMemoryStream;
begin
  Screen.Cursor := crHourGlass;
  TRY
    Bitmap.Free;
    Bitmap := TBitmap.Create;
    Bitmap.Width  := JPEGImage.Width;
    Bitmap.Height := JPEGImage.Height;

    Bitmap.PixelFormat := pf24Bit;       // force this for now

//    IF   JPEGImage.PixelFormat = jf24Bit     // Must be jf24Bit or jf8Bit
//    THEN Bitmap.PixelFormat := pf24Bit
//    ELSE Bitmap.PixelFormat := pf8Bit;

    // Draw on Canvas to convert
    Bitmap.Canvas.Draw(0,0, JPEGImage);
    ImageBMP.Picture.Graphic := Bitmap;

    // Write BMP to memory stream as quick-and-dirty way to get BMP size
    MemoryStream := TMemoryStream.Create;
    TRY
      Bitmap.SaveToStream(MemoryStream);
      BMPBytes := MemoryStream.Position;
    FINALLY
      MemoryStream.Free;
    END;

    LabelBMPBytes.Caption := FormatFloat('#,', BMPBytes) + ' bytes';

    IF   (JPGBytes > 0) AND (BMPBytes > 0)
    THEN BEGIN
      LabelCompressionFactorValue.Caption :=
        Format('%5.1f', [BMPBytes / JPGBytes]);
       LabelCompressionFactorValue.Visible := TRUE;
    END
    ELSE LabelCompressionFactorValue.Visible := FALSE;

    // Make BMP filename to match JPG except for extension
    FileName := ChangeFileExt(EditJPG.Text, '.BMP');
    EditBMP.Text := FileName;

    ButtonSaveBMP.Enabled := TRUE;
    ButtonPrintBMP.Enabled := TRUE;

    LabelBMPInfo.Caption := GetBitmapDimensionsString(Bitmap);
  FINALLY
    Screen.Cursor := crDefault
  END

end;

procedure TFormConvert.FormDestroy(Sender: TObject);
begin
  Bitmap.Free;
  JPEGImage.Free
end;





end.
