{: A simple utility that allows combining RGB and Alpha channel into a single
   32 bits texture, also allows to view RGB & Alpha channel of a 32 bits texture.<br>
   The implementation isn't high performance, just sufficiently fast for
   interactive use.<p>

   Eric Grange / GLScene<br>
   http://glscene.org
}
unit gratex;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, GLObjects, GLHUDObjects, GLScene, GLMisc, GLUtils, ExtCtrls,
  GLWin32Viewer, ComCtrls, ActnList, Menus, ImgList,
  JPeg, TGA, PNGImage, ToolWin, ExtDlgs, StdCtrls, Buttons;

type TIntegerArray = array [0..0] of Integer;
type PIntegerArray = ^TIntegerArray;


type
  TgraTTB = class(TForm)
    MainMenu: TMainMenu;
    ImageList: TImageList;
    ActionList: TActionList;
    ACExit: TAction;
    Exit1: TMenuItem;
    PAImages: TPanel;
    PAPreview: TPanel;
    Splitter1: TSplitter;
    PageControl: TPageControl;
    TSRGB: TTabSheet;
    TSAlpha: TTabSheet;
    GLSceneViewer: TGLSceneViewer;
    GLScene: TGLScene;
    GLCamera: TGLCamera;
    GLDummyCube: TGLDummyCube;
    GLCube: TGLCube;
    GLLightSource: TGLLightSource;
    HSBkgnd: TGLHUDSprite;
    ACImport: TAction;
    ACOpenTexture: TAction;
    ScrollBox1: TScrollBox;
    IMRGB: TImage;
    ScrollBox2: TScrollBox;
    IMAlpha: TImage;
    OpenPictureDialog: TOpenPictureDialog;
    Panel1: TPanel;
    Label1: TLabel;
    CBWidth: TComboBox;
    Label2: TLabel;
    CBHeight: TComboBox;
    Exit2: TMenuItem;
    ACSaveTexture: TAction;
    SaveDialog: TSaveDialog;
    SaveTexture1: TMenuItem;
    Panel2: TPanel;
    CBTextureFiltering: TCheckBox;
    CBBackground: TComboBox;
    Help1: TMenuItem;
    About1: TMenuItem;
    ToolBar: TToolBar;
    TBImport: TToolButton;
    CreateRings1: TMenuItem;
    EDTotalEclipseRadius: TEdit;
    Label3: TLabel;
    CreateShadow1: TMenuItem;
    RingImage: TImage;
    RingCB: TCheckBox;
    RingCB2: TCheckBox;
    RingBtn: TSpeedButton;
    procedure PAPreviewResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ACImportExecute(Sender: TObject);
    procedure CBWidthChange(Sender: TObject);
    procedure CBTextureFilteringClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ACOpenTextureExecute(Sender: TObject);
    procedure ACSaveTextureExecute(Sender: TObject);
    procedure GLSceneViewerMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure GLSceneViewerMouseMove(Sender: TObject; Shift: TShiftState;
      X, Y: Integer);
    procedure CBBackgroundChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Help1Click(Sender: TObject);
    procedure About1Click(Sender: TObject);
    procedure CreateRings1Click(Sender: TObject);
    procedure CreateShadow1Click(Sender: TObject);
    procedure RingBtnClick(Sender: TObject);

  private
    { Private declarations }
    mx, my : Integer;

    procedure ResetAlpha;
    function SpawnBitmap : TBitmap;
    procedure ResizeImage(im : TImage);
    procedure NormalizeAlpha;
    procedure TextureChanged;
    procedure BreakupTexture(bmp : TBitmap);

  public
    { Public declarations }
  end;

var
  graTTB: TgraTTB;

implementation

uses GLTexture,TongGlobals,vectorGeometry;

{$R *.dfm}
procedure TgraTTB.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  graTTBY := graTTB.top;
  graTTBX := graTTB.left;
end;

procedure TgraTTB.FormCreate(Sender: TObject);
begin
  top := graTTBY;
  left := graTTBX;
  CBBackground.Itemindex:=0;
  CBWidth.Itemindex:=8;
  CBHeight.Itemindex:=8;
  ResetAlpha;
  Randomize;
end;

procedure TgraTTB.Help1Click(Sender: TObject);
begin
    Application.HelpContext(7430);
end;

procedure TgraTTB.About1Click(Sender: TObject);
begin
  ShowMessage('cloned from GLScene demo by Eric Grange');
end;

procedure TgraTTB.FormShow(Sender: TObject);
begin
   PAPreviewResize(Self);
end;

procedure TgraTTB.PAPreviewResize(Sender: TObject);
const
   cTileSize = 32;
var
   w, h : Integer;
begin
   // adjust background, we could just have made huge one,
   // but that would have been too simple for a demo ;)
   w:=(GLSceneViewer.Width div cTileSize);
   h:=(GLSceneViewer.Height div cTileSize);
   HSBkgnd.XTiles:=w;
   HSBkgnd.YTiles:=h;
   w:=w*cTileSize+cTileSize;
   h:=h*cTileSize+cTileSize;
   HSBkgnd.Width:=w;
   HSBkgnd.Height:=h;
   HSBkgnd.Position.SetPoint(w div 2, h div 2, 0);
   // zoom scene with viewer's width
   GLCamera.SceneScale:=GLSceneViewer.Width/120;
end;

procedure TgraTTB.ACImportExecute(Sender: TObject);
begin
   if OpenPictureDialog.Execute then begin
      if PageControl.ActivePage=TSRGB then begin
         IMRGB.Picture.LoadFromFile(OpenPictureDialog.FileName);
         ResizeImage(IMRGB);
      end else begin
         IMAlpha.Picture.LoadFromFile(OpenPictureDialog.FileName);
         ResizeImage(IMAlpha);
         NormalizeAlpha;
      end;
      TextureChanged;
   end;
end;

function TgraTTB.SpawnBitmap : TBitmap;
begin
   Result:=TBitmap.Create;
   Result.PixelFormat:=pf32bit;
   Result.Width:=StrToInt(CBWidth.Text);
   Result.Height:=StrToInt(CBHeight.Text);
end;

procedure TgraTTB.ResetAlpha;
var
   bmp : TBitmap;
begin
   // Opaque alpha channel
   bmp:=SpawnBitmap;
   try
      with bmp.Canvas do begin
         Brush.Color:=clWhite;
         FillRect(Rect(0, 0, bmp.Width, bmp.Height));
      end;
      IMAlpha.Picture.Bitmap:=bmp;
   finally
      bmp.Free;
   end;
end;

procedure TgraTTB.NormalizeAlpha;
var
   col : Byte;
   x, y, c : Integer;
   bmp : TBitmap;
   pSrc, pDest : PIntegerArray;
begin
   bmp:=SpawnBitmap;
   try
      for y:=0 to bmp.Height-1 do begin
         pSrc:=IMAlpha.Picture.Bitmap.ScanLine[y];
         pDest:=bmp.ScanLine[y];
         for x:=0 to bmp.Width-1 do begin
            c:=pSrc[x];
            col:=Round(0.3*(c and $FF)+0.59*((c shr 8) and $FF)+0.11*((c shr 16) and $FF));
            pDest[x]:=col+(col shl 8)+(col shl 16);
         end;
      end;
      IMAlpha.Picture.Bitmap:=bmp;
   finally
      bmp.Free;
   end;
end;

procedure TgraTTB.ResizeImage(im : TImage);
var
   bmp : TBitmap;
begin
   if im.Height=0 then Exit;
   bmp:=SpawnBitmap;
   try
      bmp.Canvas.StretchDraw(Rect(0, 0, bmp.Width, bmp.Height), im.Picture.Graphic);
      im.Picture.Bitmap:=bmp;
   finally
      bmp.Free;
   end;
end;

procedure TgraTTB.BreakupTexture(bmp : TBitmap);
var
   bmpAlpha, bmpRGB : TBitmap;
   y, x, c : Integer;
   pRGB, pAlpha, pSrc : PIntegerArray;
begin
   bmpAlpha:=SpawnBitmap;
   bmpRGB:=SpawnBitmap;
   try
      bmpAlpha.Width:=bmp.Width;
      bmpAlpha.Height:=bmp.Height;
      bmpRGB.Width:=bmp.Width;
      bmpRGB.Height:=bmp.Height;
      for y:=0 to bmp.Height-1 do begin
         pRGB:=bmpRGB.ScanLine[y];
         pAlpha:=bmpAlpha.ScanLine[y];
         pSrc:=bmp.ScanLine[y];
         for x:=0 to bmp.Width-1 do begin
            c:=pSrc[x];
            pRGB[x]:=(c and $FFFFFF);
            c:=(c shr 24) and $FF;
            pAlpha[x]:=c+(c shl 8)+(c shl 16);
         end;
      end;
      IMRGB.Picture.Bitmap:=bmpRGB;
      IMAlpha.Picture.Bitmap:=bmpAlpha;
   finally
      bmpAlpha.Free;
      bmpRGB.Free;
   end;

end;

procedure TgraTTB.TextureChanged;
var
   bmp : TBitmap;
   y, x : Integer;
   pRGB, pAlpha, pDest : PIntegerArray;
begin
   if IMRGB.Picture.Graphic.Empty then Exit;
   if IMAlpha.Picture.Height=0 then begin
      GLCube.Material.Texture.Assign(IMRGB.Picture);
   end else begin
      bmp:=SpawnBitmap;
      try
         for y:=0 to bmp.Height-1 do begin
            pRGB:=IMRGB.Picture.Bitmap.ScanLine[y];
            pAlpha:=IMAlpha.Picture.Bitmap.ScanLine[y];
            pDest:=bmp.ScanLine[y];
            for x:=0 to bmp.Width-1 do
               pDest[x]:=pRGB[x] or ((pAlpha[x] and $FF) shl 24);
         end;
         GLCube.Material.Texture.Assign(bmp);
      finally
         bmp.Free;
      end;
   end;
end;

procedure TgraTTB.CBWidthChange(Sender: TObject);
begin
   ResizeImage(IMRGB);
   ResizeImage(IMAlpha);
end;

procedure TgraTTB.CBTextureFilteringClick(Sender: TObject);
begin
   with GLCube.Material.Texture do begin
      if CBTextureFiltering.Checked then begin
         MagFilter:=maLinear;
         MinFilter:=miLinearMipmapLinear;
      end else begin
         MagFilter:=maNearest;
         MinFilter:=miNearest;
      end;
   end;
end;

procedure TgraTTB.ACOpenTextureExecute(Sender: TObject);
var
  pic : TPicture;
  PNG:TPNGObject;
begin
  if OpenPictureDialog.Execute then
  begin
    pic:=TPicture.Create;
    try
      if (lowercase(extractfileext(OpenPictureDialog.FileName)) = '.png') then
      begin
        PNG := TPNGObject.Create;
        {In case something goes wrong, free both PNG and Bitmap}
        try
          PNG.LoadFromFile(OpenPictureDialog.FileName);
          pic.Bitmap.Assign(PNG); //Convert data into bitmap
        finally
          PNG.Free;
        end;
      end else pic.LoadFromFile(OpenPictureDialog.FileName);
      if (pic.Graphic is TBitmap) and (pic.Bitmap.PixelFormat=pf32bit) then
      begin
        BreakupTexture(pic.Bitmap);
        ResizeImage(IMAlpha);
      end else
      begin
        IMRGB.Picture:=pic;
        ResetAlpha;
      end;
      ResizeImage(IMRGB);
      TextureChanged;
    finally
      pic.Free;
    end;
  end;
end;

procedure TgraTTB.ACSaveTextureExecute(Sender: TObject);
var
   pic : TPicture;
   fName : String;
   tga : TTGAImage;
   PNG:TPNGObject;
begin
   pic:=(GLCube.Material.Texture.Image as TGLPictureImage).Picture;
   if (pic.Height>0) and SaveDialog.Execute then begin
      fName:=SaveDialog.FileName;
      if ExtractFileExt(fName)='' then
         if SaveDialog.FilterIndex=1 then
            fName:=fName+'.bmp'
         else if SaveDialog.FilterIndex=2 then fName:=fName+'.tga'
         else  fName:=fName+'.png';
      if LowerCase(ExtractFileExt(fName))='.png' then begin
        PNG := TPNGObject.Create;
        PNG.CreateAlpha;
        {In case something goes wrong, free both PNG and Bitmap}
        try
          PNG.Assign(pic.Bitmap); //Convert data into png
          PNG.SaveToFile(fName);
        finally
          PNG.Free;
        end;
      end else
      if LowerCase(ExtractFileExt(fName))='.tga' then begin
         tga:=TTGAImage.Create;
         try
            tga.Assign(pic.Bitmap);
            tga.SaveToFile(fName)
         finally
            tga.Free;
         end;
      end else pic.SaveToFile(fName);
   end;
end;

procedure TgraTTB.GLSceneViewerMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
   mx:=x; my:=y;
end;

procedure TgraTTB.GLSceneViewerMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
   if Shift=[ssLeft] then begin
      GLCamera.MoveAroundTarget(my-y, mx-x);
      mx:=x; my:=y;
   end;
end;

procedure TgraTTB.CBBackgroundChange(Sender: TObject);
begin
   HSBkgnd.Visible:=(CBBackground.ItemIndex=0);
   case CBBackground.ItemIndex of
      1 : GLSceneViewer.Buffer.BackgroundColor:=clBlack;
      2 : GLSceneViewer.Buffer.BackgroundColor:=clSilver;
      3 : GLSceneViewer.Buffer.BackgroundColor:=clWhite;
   end;
end;



function DiskArcSurface(diskRadius, sliceAngleRadians : Single) : Single;
begin
   Result:=Sqr(diskRadius)*( sliceAngleRadians        // whole slice surface
                            -0.5*Sin(2*sliceAngleRadians)); // slice triangle surface
end;

procedure DiskDiskIntersect(disk1Radius, disk2Radius, centersDistance : Single;
                            var disk1Alpha, disk2Alpha : Single);
var
   px : Single;
begin
   {  px+py = r1
      (cd-px)+py = r2  ->  cd - 2.cd.px + px + py = r2
                          ->  px = (r1 + cd - r2)/2.cd }
   px:=(Sqr(disk1Radius)+Sqr(centersDistance)-Sqr(disk2Radius))/(2*centersDistance);
   disk1Alpha:=ArcCos(px/disk1Radius);
   disk2Alpha:=ArcCos((centersDistance-px)/disk2Radius);
end;

function VisibleSurface(diskRadius, occultingDiskRadius, occultingDiskCenterDelta : Single) : Single;
var
   u, r0Pr1, r0Mr1, diskAlpha, occludingAlpha : Single;
begin
   u:=occultingDiskCenterDelta;
   r0Pr1:=diskRadius+occultingDiskRadius;
   r0Mr1:=Abs(diskRadius-occultingDiskRadius);
   Result:=PI*Sqr(diskRadius);
   if u>=r0Pr1 then
      Exit
   else if u>r0Mr1 then begin
      DiskDiskIntersect(diskRadius, occultingDiskRadius, occultingDiskCenterDelta,
                        diskAlpha, occludingAlpha);
      Result:= Result
              -DiskArcSurface(diskRadius, diskAlpha)
              -DiskArcSurface(occultingDiskRadius, occludingAlpha);
   end else Result:=Result-PI*Sqr(MinFloat(diskRadius, occultingDiskRadius));
end;

function Penumbra(radius, teRadius : Single) : Single;
var
   sunRadius, moonRadius, centerDelta : Single;
begin
{   if radius>teRadius then begin
      if radius>1 then
         Result:=1
      else begin }
         moonRadius:=1;
         //
         sunRadius:=moonRadius-teRadius*(1/400);
         centerDelta:=(moonRadius+sunRadius)*(radius-teRadius)/(1-teRadius);
         Result:=VisibleSurface(sunRadius, moonRadius, centerDelta);
         Result:=Result/(PI*Sqr(sunRadius));
{      end;
   end else Result:=0;}
end;


procedure TgraTTB.CreateRings1Click(Sender: TObject);
var
  pic   : TPicture;
  Spawn : TBitmap;
  x2, y : Integer;
  col : TColor;
begin
  CBBackground.Itemindex:=0;
  CBWidth.Itemindex:=8;
  CBHeight.Itemindex:=8;
  pic:=TPicture.Create;
  Spawn := SpawnBitmap;  
  ResetAlpha;
  try
   x2:=0;
   for y:=0 to 255{127} do begin
   If ((RingCB.Checked)and (RingCB2.Checked)) then
   col:=(RingImage.Canvas.Pixels[x2, 0]+random(32)-random(32)) else
   If RingCB.Checked then
   col:=RingImage.Canvas.Pixels[x2, 0] else
   col:=random(16777216);
   inc(x2);
    Spawn.Canvas.pen.Color := col;
    Spawn.Canvas.Brush.Color := col;
    Spawn.Canvas.Brush.Style := bssolid;
    Spawn.Canvas.Ellipse(
    0+x2-127,
    0+x2-127,
    Spawn.Width-x2+127,
    Spawn.Height-x2+127);
   end;
  pic.Bitmap.Assign(Spawn);
  BreakupTexture(pic.Bitmap);
  ResizeImage(IMAlpha);
  ResizeImage(IMRGB);
  TextureChanged;
  finally
      pic.Free;
      Spawn.Free;
  end;
end;



procedure TgraTTB.CreateShadow1Click(Sender: TObject);
var
  pic   : TPicture;
  Spawn : TBitmap;
  x,y : Integer;
   col : TColor;
   teRadius, r, intensity : Single;
begin
  CBBackground.Itemindex:=0;
  CBWidth.Itemindex:=8;
  CBHeight.Itemindex:=8;
  pic:=TPicture.Create;
  Spawn := SpawnBitmap;
  ResetAlpha;
  try
   teRadius:=StrToFloatDef(EDTotalEclipseRadius.Text, 0);

   for y:=0 to 127 do begin
      for x:=0 to 127 do begin
         r:=Sqrt(Sqr(127.5-x)+Sqr(127.5-y))*(1/126.5);
         intensity:=Penumbra(r, teRadius);
         col:=RGB(
         Round(intensity*((200)+Random(55))),
         Round(intensity*((200)+Random(55))),
         Round(intensity*((200)+Random(55))));
         with Spawn.Canvas do begin
            Pixels[x, y]:=col;
            Pixels[x, 255-y]:=col;
            Pixels[255-x, y]:=col;
            Pixels[255-x, 255-y]:=col;
         end;
      end;
   end;
  //pic.Canvas.draw(0,0,Spawn);
  pic.Bitmap.Assign(Spawn);
  BreakupTexture(pic.Bitmap);
  ResizeImage(IMAlpha);
  ResizeImage(IMRGB);
  TextureChanged;
  finally
      pic.Free;  Spawn.Free;
  end;
end;

procedure TgraTTB.RingBtnClick(Sender: TObject);
begin
  if OpenPictureDialog.Execute then
  begin
    if (lowercase(extractfileext(OpenPictureDialog.FileName)) = '.bmp') then
    RingImage.Picture.Bitmap.LoadFromFile(OpenPictureDialog.FileName);
  end;
end;


end.
